//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.assistant;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.bcee.assistant.BceeCreateAccountAssistent;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.bsh.assistant.BshCreateAccountAssistent;
import de.frankmuenster.hibiscus.mahoe.santander.SantanderBank;
import de.frankmuenster.hibiscus.mahoe.santander.assistant.SantanderCreateAccountAssistant;
import de.frankmuenster.hibiscus.mahoe.targobank.TargoBank;
import de.frankmuenster.hibiscus.mahoe.targobank.assistant.TargoBankCreateAccountAssistent;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.jameica.system.OperationCanceledException;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Ein Assistent, der bei der Kontoerstellung hilft.
 * 
 * @author Frank Mnster
 * 
 */
public final class CreateAccountAssisstant implements Action {

	/**
	 * Standardkonstruktor
	 */
	public CreateAccountAssisstant() {
		// absichtlich leer
	}

	/**
	 * @see de.willuhn.jameica.gui.Action#handleAction(java.lang.Object)
	 */
	@Override
	public void handleAction(final Object context) throws ApplicationException {

		// Erst mal die Bank heraus finden
		final CreateKontoStartDialog startDialog = new CreateKontoStartDialog();
		try {
			final String blz = startDialog.open();
			createAccount(blz);
		} catch (OperationCanceledException e) {
			Logger.info(I18N.tr("CreateAccountAssisstant.cancelledByUser")); //$NON-NLS-1$
		} catch (Exception e) { // NOPMD
			Logger.error(I18N.tr("CreateAccountAssisstant.openDialogFailed", e.getMessage()), e); //$NON-NLS-1$
		}

	}

	/**
	 * berprft ob schon ein MahoeMain Konto existiert und started ggf. den
	 * Kontoanlage-Assistenten.
	 * 
	 * @param blz
	 */
	private static void createAccount(final String blz) {

		final String bankName = MahoeSynchronizeBackend.BLZ_SUPPORTED.get(blz);
		InterfaceCreateAccountAssistant createAssistent = null;
		switch (blz) {
		case TargoBank.BLZ:
			createAssistent = new TargoBankCreateAccountAssistent();
			break;
		case Bcee.BLZ:
			createAssistent = new BceeCreateAccountAssistent();
			break;
		case SchwaebischHall.BLZ:
			createAssistent = new BshCreateAccountAssistent();
			break;
		case SantanderBank.BLZ:
			createAssistent = new SantanderCreateAccountAssistant();
			break;
		default:
			break;
		}
		if (createAssistent != null) {
			try {
				final Konto konto = createAssistent.createAccount();
				GUI.startView(de.willuhn.jameica.hbci.gui.views.KontoNew.class, konto);
				final StatusBarMessage statusMsg = new StatusBarMessage(I18N.tr(
						"CreateAccountAssisstant.createAccountSuccessful", bankName), //$NON-NLS-1$
						StatusBarMessage.TYPE_SUCCESS);
				Application.getMessagingFactory().sendMessage(statusMsg);
			} catch (CreateAccountException e) {
				Logger.error(I18N.tr("CreateAccountAssisstant.createAccountFailed", e.getMessage()), e); //$NON-NLS-1$
				// Benutzer ber Fehler beim Anlegen des Kontos benachrichtigen
				final StatusBarMessage statusMsg = new StatusBarMessage(I18N.tr(
						"CreateAccountAssisstant.createAccountFailedStatusBarMsg", bankName), //$NON-NLS-1$
						StatusBarMessage.TYPE_ERROR);
				Application.getMessagingFactory().sendMessage(statusMsg);
				try {
					final String msg = I18N.tr("CreateAccountAssisstant.createAccountFailedPopupMsg"); //$NON-NLS-1$
					Application.getCallback().notifyUser(msg);
				} catch (Exception e1) { // NOPMD
					// Nur loggen;
					Logger.error(I18N.tr("CreateAccountAssisstant.userInfoFailed", e.getMessage()), e); //$NON-NLS-1$
				}
			}

		}
	}

}
