//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bcee.web; // NOPMD 

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.ScriptResult;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.HtmlAnchor;
import com.gargoylesoftware.htmlunit.html.HtmlButton;
import com.gargoylesoftware.htmlunit.html.HtmlInput;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTableBody;
import com.gargoylesoftware.htmlunit.html.HtmlTableCell;
import com.gargoylesoftware.htmlunit.html.HtmlTableRow;

import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaverException;
import de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * BCEE Bank Website
 * 
 * @author Frank Mnster
 * 
 */
public final class BceeWebSite extends AbstractWebSite<BceeAccountInformation, BceeBankMessages> {

	/**
	 * Unsere BceeWebSite instance
	 */
	private static BceeWebSite webSite;

	/**
	 * Die LuxTrust User ID
	 */
	private final transient String luxTrustUserId;

	/**
	 * TransactionImpl Objekt
	 */
	private final transient BceeWebSiteTransactionsImpl transImpl;

	/**
	 * Liefert die Instance der WebSeite
	 * 
	 * @param konto
	 * @return die WebSeite
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	public static BceeWebSite getInstance(final Konto konto) throws WebSiteException, ApplicationException {
		String luxTrustUser = null;
		try {
			luxTrustUser = konto.getKundennummer();
		} catch (RemoteException e) {
			final String msg = I18N.tr("BceeWebSite.loginMissing"); // NOPMD //$NON-NLS-1$
			Logger.error(msg, e);
			throw new ApplicationException(msg, e);
		}

		if (BceeWebSite.webSite == null) { // NOPMD
			BceeWebSite.webSite = new BceeWebSite(konto, luxTrustUser);
		} else {
			// Ist die LuxTrustUserId dieselbe
			if (BceeWebSite.webSite.luxTrustUserId.equals(luxTrustUser)) {
				// Aktualisiere Startdatum der Umsatzabfrage und lsche
				// Umsatzliste
				final Date lastTransacDate = Utils.getLastUmsatzDate(konto, BceeWebSite.webSite.getStartdate());
				BceeWebSite.webSite.accountInfo.setLastTransactionDate(lastTransacDate);
				BceeWebSite.webSite.transImpl.clear();
			} else {
				BceeWebSite.webSite = new BceeWebSite(konto, luxTrustUser);
			}
		}
		return BceeWebSite.webSite;
	}

	/**
	 * Erstellt ein neues Webseiten-Objekt zur Kontaktaufnahme mit der BCEE.
	 * 
	 * @param konto
	 * @param luxTrustUserId
	 * @throws WebSiteException
	 * 
	 */
	private BceeWebSite(final Konto konto, final String luxTrustUserId) throws WebSiteException {
		super(konto);
		this.luxTrustUserId = luxTrustUserId;
		this.webClient = BceeWebSiteUtils.getWebClient();
		this.transImpl = new BceeWebSiteTransactionsImpl();
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
	 */
	@Override
	public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung) throws WebSiteException,
			ApplicationException {

		BceeWebSiteTransferImpl.createSepaTransfer(this.webClient, aUeberweisung);
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#getAccountInformation()
	 */
	@Override
	public BceeAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

		// Lese Saldo
		this.getAccountInfo();

		final HtmlPage page = BceeWebSiteUtils.openAccount(this.webClient, this.accountInfo);
		final HtmlAnchor infoButton = (HtmlAnchor) page.getElementById("informationButton"); //$NON-NLS-1$

		try {
			final HtmlPage infoPage = infoButton.click();

			Logger.debug(infoPage.asText());
			// Logger.debug(infoPage.asXml());

			HtmlTable table;
			do {
				try {
					Thread.sleep(500);
				} catch (InterruptedException e) {
					// Nix zu tun hier;
				}
				table = (HtmlTable) infoPage.getElementById("popupTable"); //$NON-NLS-1$
				Logger.debug("Infopage still loading..."); //$NON-NLS-1$
			} while (table == null);

			Logger.debug(table.asText());

			final HtmlTableBody tbBody = table.getBodies().get(0);

			String limit = StringUtils.EMPTY;
			String interestRate = StringUtils.EMPTY;
			for (int i = 0; i < tbBody.getRows().size(); i++) {

				final HtmlTableRow row = tbBody.getRows().get(i);
				if (row.getCell(0).asText().contains("Kreditlinie")) { //$NON-NLS-1$
					limit = tbBody.getRows().get(i + 1).getCells().get(1).asText();
					interestRate = tbBody.getRows().get(i + 1).getCells().get(1).asText();
					break;
				}

			}

			Logger.debug(interestRate);
			Logger.debug(limit);

			if (!interestRate.isEmpty() && !limit.isEmpty()) {
				this.accountInfo.setInterestRate(Utils.convertStringToDouble(interestRate.replaceAll(
						"%", StringUtils.EMPTY))); //$NON-NLS-1$
				this.accountInfo.setCreditLimit(Utils.convertStringToDouble(limit));
			}

		} catch (IOException e) {
			Logger.error(I18N.tr("BceeWebSite.loadAccountDetailsFailed"), e); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSite.loadAccountDetailsFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
		}

		return this.accountInfo;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#getBookedTransactions()
	 */
	@Override
	public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {
		return this.transImpl.getBookedTransactions(this.webClient, this.accountInfo);
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#getPreBookedTransactions()
	 */
	@Override
	public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {
		return this.transImpl.getPreBookTransactions(this.webClient, this.accountInfo);
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#isStillLoggedIn()
	 */
	@Override
	public boolean isStillLoggedIn() throws WebSiteException {

		boolean isStillLoggedin = false;

		try {
			final HtmlPage page = this.webClient.getPage(Bcee.URL_HOME);
			if (page.asText().contains("Willkommen")) { //$NON-NLS-1$
				isStillLoggedin = true;
			}
		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(I18N.tr("BceeWebSite.connectionCheckFailed"), e); //$NON-NLS-1$
		}

		return isStillLoggedin;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#login(char[],
	 *      java.lang.String[])
	 */
	@Override
	public void login(final char[] password, final String... loginParams) throws WebSiteException {
		Logger.debug(I18N.tr("BceeWebSite.startLogin") + this.luxTrustUserId); //$NON-NLS-1$

		if (loginParams.length != 1) { // NOPMD
			throw new WebSiteException(I18N.tr("BceeWebSite.loginParameterCountWrong"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_ERROR);
		}

		// Get Login Page
		HtmlPage page = this.getLoginPage();

		// Anmelden
		Logger.debug("Inspiziere Webseite und hole Formularfelder"); //$NON-NLS-1$
		final DomElement userIdInput = page.getElementById("user-id"); //$NON-NLS-1$
		final DomElement passwortInput = page.getElementById("password"); //$NON-NLS-1$
		final DomElement optInput = page.getElementById("otp"); //$NON-NLS-1$

		// Set Values
		Logger.info(I18N.tr("BceeWebSite.setLoginData") + this.luxTrustUserId); //$NON-NLS-1$
		((HtmlInput) userIdInput).setValueAttribute(this.luxTrustUserId);

		try {
			((HtmlInput) passwortInput).focus();
			for (final char c : password) {
				page = (HtmlPage) ((HtmlInput) passwortInput).type(c);
			}

			((HtmlInput) optInput).focus();
			for (final char c : loginParams[0].toCharArray()) {
				page = (HtmlPage) ((HtmlInput) optInput).type(c);
			}

			// Authentication Button
			final HtmlButton submit = page
					.getFirstByXPath("/html/body/div/div/div/div/fieldset/fieldset/div/button[2]"); //$NON-NLS-1$

			page = submit.click();
			Logger.debug(page.asText());

		} catch (IOException e) {
			Logger.error(I18N.tr("BceeWebSite.settingDataFailed"), e); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSite.settingDataFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_FAILED, e);
		}

		// Anmeldung berprfen
		this.checkLogin(page);

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#logoff()
	 */
	@Override
	public void logoff() {
		// TODO Auto-generated method stub
		Logger.info("Schliee Bankverbindung. (Noch nicht implementiert)"); //$NON-NLS-1$

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#savePDF(de.frankmuenster.hibiscus.mahoe.web.InterfaceWebAccountInformation)
	 */
	@Override
	public int savePDF(final BceeAccountInformation accountInfo) throws PdfSaverException {
		Logger.info("Speichern von Kontoauszgen wird noch nicht untersttzt."); //$NON-NLS-1$
		return 0;
	}

	/**
	 * &Uuml;berpr&uuml;pft die Loginseite, ob der Login erfolgreich war.
	 * 
	 * @param page
	 * @throws WebSiteException
	 *             falls der Login nicht erfolgreich war.
	 */
	private void checkLogin(final HtmlPage page) throws WebSiteException {
		final String pageText = page.asText();
		Logger.debug(pageText);

		// Click Result - TEST FOR ERR..
		if (pageText.contains("fen Sie die Eingabe der User ID")) { //$NON-NLS-1$
			Logger.error(I18N.tr("BceeWebSite.idInvalidText")); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSite.idInvalidText"), WebSiteExceptionTypes.LOGIN_FAILED); //$NON-NLS-1$
		} else if (pageText.contains("fen Sie Ihr pers")) { //$NON-NLS-1$
			Logger.error(I18N.tr("BceeWebSite.passwordInvalid")); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSite.passwordInvalid"), WebSiteExceptionTypes.LOGIN_FAILED); //$NON-NLS-1$
		} else if (pageText.contains("LuxTrust Passwort sind unzutreffend")) { //$NON-NLS-1$
			Logger.error(I18N.tr("BceeWebSite.idOrPasswordMalformed")); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSite.idOrPasswordMalformed"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_FAILED);
		} else if (pageText.contains("OTP (6 Zeichen)")) { //$NON-NLS-1$
			Logger.error(I18N.tr("BceeWebSite.otpInvalid")); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSite.otpInvalid"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_FAILED);
		} else if (pageText.contains("Ein paar Daten sind nicht korrekt")) { //$NON-NLS-1$
			Logger.error(I18N.tr("BceeWebSite.dataIncorrect")); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSite.dataIncorrect"), WebSiteExceptionTypes.LOGIN_FAILED); //$NON-NLS-1$
		}

		// And again test for error
		if (pageText.contains("Datum der letzten Verbindung")) { //$NON-NLS-1$
			Logger.info(I18N.tr("BceeWebSite.loginSuccessful")); //$NON-NLS-1$
		} else {
			Logger.error(I18N.tr("BceeWebSite.loginFailed")); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSite.loginFailedunknownReason"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_FAILED);
		}

	}

	/**
	 * List das Saldo und anderer Dinge und speichert sie in dem
	 * accountInfo-Object
	 * 
	 * @throws WebSiteException
	 *             falls irgendwas schief geht
	 */
	private void getAccountInfo() throws WebSiteException {

		// ffne Konto
		HtmlPage page;
		Boolean accountFound = false;

		try {
			page = this.webClient.getPage(Bcee.URL_UEBERSICHT);
			Logger.debug(page.asText());

			if (page.asText().contains("Finanzlage : Strukturierte Darstellung")) { //$NON-NLS-1$

				// Tabelle mit Konten
				final HtmlTable table = (HtmlTable) page.getElementById("tableSorterExtended"); //$NON-NLS-1$
				final HtmlTableBody tbBody = table.getBodies().get(0);

				for (final HtmlTableRow row : tbBody.getRows()) {
					Logger.debug(row.getCells().get(0).getFirstChild().toString());

					if (row.getCells().get(0).getFirstChild().toString().equals("Haben")) { //$NON-NLS-1$
						continue;
					}

					final String iban = row.getCells().get(1).asText().replaceAll("\\s", StringUtils.EMPTY); //$NON-NLS-1$
					Logger.debug("Vorhandene Konten: " + iban); //$NON-NLS-1$
					Logger.debug("Konto IBAN: " + this.accountInfo.getKonto().getIban()); //$NON-NLS-1$

					if (iban.equals(this.accountInfo.getKonto().getIban())) {
						accountFound = true;

						final List<HtmlTableCell> cells = row.getCells();
						// Saldo auslesen
						final String saldo = cells.get(5).asText();
						this.accountInfo.setSaldo(Utils.convertStringToDouble(saldo));

						// Whrung
						final String currency = cells.get(4).asText();
						this.accountInfo.setCurrency(currency);

						break;
					}
				}
			}
		} catch (FailingHttpStatusCodeException | IOException e) {
			final WebSiteException wse = new WebSiteException(e.getMessage(), WebSiteExceptionTypes.KONTO_NOT_FOUND);
			wse.addSuppressed(e);
			throw wse; // NOPMD
		}
		if (!accountFound) {
			// Wenn man hier ist, wurde das Konto nicht gefunden
			Logger.error(I18N.tr("BceeWebSite.accountOverviewNotFound")); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("BceeWebSite.accountOverviewNotFound"), //$NON-NLS-1$
					WebSiteExceptionTypes.KONTO_NOT_FOUND);
		}
	}

	/**
	 * Liefert die Loginseite, nach Auswahl von LuxTrust und Token.
	 * 
	 * @return die eigentliche Loginseite.
	 * @throws WebSiteException
	 */
	private HtmlPage getLoginPage() throws WebSiteException {

		HtmlPage page;

		try {
			Logger.debug("\u00D6ffne login Seite"); //$NON-NLS-1$
			page = this.webClient.getPage(Bcee.URL_LOGIN);

			Logger.debug("\u00D6ffen LuxTrust-auswahl"); //$NON-NLS-1$
			String javaScriptCode = "selectLuxtrust('de_DE');"; //$NON-NLS-1$
			ScriptResult result = page.executeJavaScript(javaScriptCode);
			page = (HtmlPage) result.getNewPage();

			Logger.debug(page.asText());

			Logger.debug("\u00D6ffen Tokenseite"); //$NON-NLS-1$
			javaScriptCode = "chooseProduct('token')"; //$NON-NLS-1$
			result = page.executeJavaScript(javaScriptCode);
			page = (HtmlPage) result.getNewPage();

			Logger.debug(page.asText());

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.debug(e.getMessage());
			Logger.error(I18N.tr("BceeWebSite.openStartPageFailedLog")); //$NON-NLS-1$
			throw new WebSiteException(
					I18N.tr("BceeWebSite.openStartPageFailed"), WebSiteExceptionTypes.PAGE_NOT_FOUND, //$NON-NLS-1$
					e);
		}
		return page;

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite#getStartdate()
	 */
	@Override
	protected Calendar getStartdate() {
		// Setze StartDatum auf heute - 2 Jahre
		final Calendar startCalendar = Calendar.getInstance();
		startCalendar.add(Calendar.MONTH, -23);
		return startCalendar;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite#getWebClient()
	 */
	@Override
	protected WebClient getWebClient() {
		return this.webClient;
	}

}
