//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bcee.web;

import java.io.IOException;
import java.rmi.RemoteException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.TextPage;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.WebResponse;
import com.gargoylesoftware.htmlunit.html.HtmlElement;
import com.gargoylesoftware.htmlunit.html.HtmlForm;
import com.gargoylesoftware.htmlunit.html.HtmlInput;
import com.gargoylesoftware.htmlunit.html.HtmlOption;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlSelect;
import com.gargoylesoftware.htmlunit.javascript.host.event.Event;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Statische Implementierungen der Umsatzfunktionen
 * 
 * @author Frank Mnster
 * 
 */
public final class BceeWebSiteTransactionsImpl {

	/**
	 * Anzahl der Felder in der CSV-Datei
	 */
	private static final int CSV_FIELDS = 11;

	/**
	 * Interne Liste mit allen Umstzen
	 */
	private transient List<Umsatz> allTransactions;

	/**
	 * Privater Konstruktor
	 */
	public BceeWebSiteTransactionsImpl() {
		this.allTransactions = new ArrayList<Umsatz>();
	}

	/**
	 * Liest alle vorgemerkten Ums&auml;tze. Als Kriterium wird das Auszug
	 * genommen. Fehlt dieser, wird der Umsatz als vorgemerkt markiert.
	 * 
	 * @param webClient
	 * @param accountInfo
	 * 
	 * @return List mit vorgemerkten Ums&auml;tzen
	 * 
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	public List<Umsatz> getPreBookTransactions(final WebClient webClient, // NOPMD
			final BceeAccountInformation accountInfo) throws WebSiteException, ApplicationException {

		if (this.allTransactions.isEmpty()) {
			this.getTransactions(webClient, accountInfo);
		}

		final List<Umsatz> transactions = new ArrayList<>();
		for (final Umsatz umsatz : this.allTransactions) {

			try {
				if (StringUtils.isEmpty(umsatz.getKommentar())) {
					umsatz.setFlags(Umsatz.FLAG_NOTBOOKED);
					transactions.add(umsatz);
				}
			} catch (RemoteException e) {
				Logger.error(I18N.tr //
				("BceeWebSiteTransactionsImpl.getTransactionTypeFailed"), e); //$NON-NLS-1$ NOPMD
				throw new WebSiteException(I18N.tr("BceeWebSiteTransactionsImpl.getTransactionTypeFailed"), e); //$NON-NLS-1$
			}
		}
		return transactions;
	}

	/**
	 * Liste die gebuchten Umstze ein.
	 * 
	 * @param webClient
	 *            der angemeldete WebClient
	 * @param accountInfo
	 *            die Kontoinformationen
	 * @return ein Liste mit den gebuchtenUmstzen
	 * 
	 * @throws WebSiteException
	 */
	public List<Umsatz> getBookedTransactions(final WebClient webClient, // NOPMD
			final BceeAccountInformation accountInfo) throws WebSiteException {

		if (this.allTransactions.isEmpty()) {
			this.getTransactions(webClient, accountInfo);
		}

		final List<Umsatz> transactions = new ArrayList<>();
		for (final Umsatz umsatz : this.allTransactions) {

			try {
				if (StringUtils.isNotEmpty(umsatz.getKommentar())) {
					transactions.add(umsatz);
				}
			} catch (RemoteException e) {
				Logger.error(I18N.tr("BceeWebSiteTransactionsImpl.getTransactionTypeFailed"), e); //$NON-NLS-1$
				throw new WebSiteException(I18N.tr("BceeWebSiteTransactionsImpl.getTransactionTypeFailed"), e); //$NON-NLS-1$
			}

		}

		return transactions;
	}

	/**
	 * 
	 * Liest alle Ums&auml;tze in die interen Liste
	 * 
	 * @param webClient
	 * @param accountInfo
	 * @throws WebSiteException
	 */
	private void getTransactions(final WebClient webClient, final BceeAccountInformation accountInfo)
			throws WebSiteException {

		final SimpleDateFormat sdfMonthName = new SimpleDateFormat("MMMM yyyy", //$NON-NLS-1$
				Locale.GERMAN);
		final SimpleDateFormat sdfMonthNumber = new SimpleDateFormat("01/MM/yyyy 00:00:00", Locale.GERMAN); //$NON-NLS-1$

		final List<Umsatz> transactions = new ArrayList<>();

		final Calendar lastMonth = Calendar.getInstance();
		lastMonth.setTime(accountInfo.getLastTransactionDate());
		lastMonth.add(Calendar.MONTH, -1);
		lastMonth.set(Calendar.DAY_OF_MONTH, 1);
		Logger.info(I18N.tr("BceeWebSiteTransactionsImpl.loadTransactionsfrom") //$NON-NLS-1$
				+ sdfMonthName.format(lastMonth.getTime()));

		final Calendar month = Calendar.getInstance();
		month.set(Calendar.DAY_OF_MONTH, month.getActualMaximum(Calendar.DAY_OF_MONTH));
		month.set(Calendar.HOUR, 0);
		month.set(Calendar.MINUTE, 0);

		HtmlPage page = BceeWebSiteUtils.openAccount(webClient, accountInfo);

		while (lastMonth.before(month)) {

			// Setzte Monat
			final HtmlForm changeDateForm = (HtmlForm) page.getElementById("changeDateForm"); //$NON-NLS-1$

			final Date date = month.getTime();
			final String monthName = sdfMonthName.format(date);
			final String monthNumber = sdfMonthNumber.format(date);

			Logger.info(I18N.tr("BceeWebSiteTransactionsImpl.loadMonth", monthNumber, monthName)); //$NON-NLS-1$

			final HtmlSelect select = (HtmlSelect) page.getElementById("SelectedMonth"); //$NON-NLS-1$
			final HtmlOption option = select.getOptionByValue(monthNumber);
			option.setSelected(true, true);
			select.setSelectedAttribute(option, true);

			final HtmlInput selMonthInput = (HtmlInput) page.getElementById("SelectedMonthInput"); //$NON-NLS-1$
			selMonthInput.setValueAttribute(monthName);
			selMonthInput.setAttribute("data-val-value", monthName); //$NON-NLS-1$
			selMonthInput.setAttribute("data-val-selectedvalue", monthName); //$NON-NLS-1$

			changeDateForm.fireEvent(Event.TYPE_SUBMIT);

			// create a submit button - it doesn't work with 'input'
			final HtmlElement submitBtn = (HtmlElement) page.createElement("button"); //$NON-NLS-1$
			submitBtn.setAttribute("type", "submit"); //$NON-NLS-1$ //$NON-NLS-2$

			changeDateForm.appendChild(submitBtn);

			try {
				final HtmlPage page1 = submitBtn.click();
				final HtmlElement input = (HtmlElement) page1.getElementById("btnExport"); //$NON-NLS-1$

				page = input.click();

				final HtmlElement button = (HtmlElement) page.getByXPath("/html/body/div/div/div/button").get(1); //$NON-NLS-1$
				final TextPage result = button.click();
				final WebResponse response = result.getWebResponse();

				if (response.getResponseHeaderValue("Content-Type").equals( //$NON-NLS-1$
						"text/csv")) { //$NON-NLS-1$
					// Bank-Antwort is UFT-16LE (Little Endian) kodiert
					final String csv = response.getContentAsString("UTF-16LE"); //$NON-NLS-1$
					this.loadCsv(csv, transactions);
				} else {
					throw new WebSiteException(I18N.tr("BceeWebSiteTransactionsImpl.exportCsvFailed"), //$NON-NLS-1$
							WebSiteExceptionTypes.PAGE_NOT_FOUND);
				}

			} catch (IOException e) {
				Logger.error(I18N.tr("BceeWebSiteTransactionsImpl.loadCsvPageFailed"), e); //$NON-NLS-1$
				throw new WebSiteException(I18N.tr("BceeWebSiteTransactionsImpl.loadCsvPageFailed"), e); //$NON-NLS-1$
			}

			month.add(Calendar.MONTH, -1);

		}

		// Salden hinzufgen und Reihenfolge umdrehen
		this.allTransactions = this.addSalden(transactions, accountInfo);

	}

	/**
	 * Fgt die Salden hinzu und dreht die Reihenfolge um, damit inHibisucs die
	 * Umsatz ID richtig ist (neue haben hhere IDs).
	 * 
	 * @param transactions
	 * @param accountInfo
	 * 
	 * @return Liste mit den gefundenen Ums&auml;tzen
	 */
	private List<Umsatz> addSalden(final List<Umsatz> transactions, final BceeAccountInformation accountInfo) {

		// Teste of Webseitensaldo und Saldo der erstes Umsatz
		double saldo = accountInfo.getSaldo();

		try {
			for (final Umsatz umsatz : transactions) {
				umsatz.setSaldo(saldo);
				saldo = Utils.roundNumber(saldo - umsatz.getBetrag(), 2);
			}
		} catch (RemoteException e) {
			Logger.error(I18N.tr("BceeWebSiteTransactionsImpl.setSaldoFailed"), e); //$NON-NLS-1$
		}

		// Liste umkehren
		final List<Umsatz> transWithSaldo = new ArrayList<>();
		for (int i = transactions.size() - 1; i >= 0; i--) {
			final Umsatz transacUmsatz = transactions.get(i);
			transWithSaldo.add(transacUmsatz);
		}

		return transWithSaldo;

	}

	/**
	 * L&auml;d die Ums&auml;tze aus der CSV-Datei in die Umsatzliste.
	 * 
	 * @param csv
	 * @param transactions
	 * 
	 * @throws RemoteException
	 */
	private void loadCsv(final String csv, final List<Umsatz> transactions) throws RemoteException {

		final String[] lines = csv.split("\r\n"); //$NON-NLS-1$

		for (int i = lines.length - 1; i >= 0; i--) {
			final String line = lines[i];
			final String[] transaction = line.split(";", -1); //$NON-NLS-1$
			if (transaction.length == BceeWebSiteTransactionsImpl.CSV_FIELDS) {
				if (transaction[0].equals("Datum")) { //$NON-NLS-1$
					continue;
				}
				final Umsatz umsatz = this.loadLine(transaction);
				transactions.add(umsatz);

			}
		}

	}

	/**
	 * L&auml;d eine Zeile aus der CSV-Datei in einen Umsatz.
	 * 
	 * @param transaction
	 *            die Umsatzzeile aus dem CSV
	 * 
	 * @return den Umsatz
	 * 
	 * @throws RemoteException
	 */
	private Umsatz loadLine(final String[] transaction) // NOPMD
			throws RemoteException {

		// Datum;Beschreibung;Valuta;Betrag in EUR;
		// Auszug;Saldo;Transaktion;Mitteilung1;Mitteilung2;Mitteilung3;Mitteilung4

		final HBCIDBService dbService = Settings.getDBService();
		final SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy", //$NON-NLS-1$
				Locale.getDefault());
		final Umsatz umsatz = dbService.createObject(Umsatz.class, null);

		// Buchungsdatum
		Date date = null;
		try {
			date = sdf.parse(transaction[0]);
		} catch (ParseException e) {
			Logger.error(I18N.tr("BceeWebSiteTransactionsImpl.parseBookingDateFailed", //$NON-NLS-1$
					e.getMessage()), e);
		}
		umsatz.setDatum(date);

		// Zweck
		final String zweck = transaction[1] + "\n" + transaction[7] + "\n" //$NON-NLS-1$ //$NON-NLS-2$
				+ transaction[8] + "\n" + transaction[9] + "\n" //$NON-NLS-1$//$NON-NLS-2$
				+ transaction[10];
		Utils.loadUmsatz(umsatz, zweck);

		// Valuta
		try {
			date = sdf.parse(transaction[2]);
		} catch (ParseException e) {
			Logger.error(I18N.tr("BceeWebSiteTransactionsImpl.parseValutaDateFailed", //$NON-NLS-1$
					e.getMessage()), e);
		}
		umsatz.setValuta(date);

		// Betrag in EUR
		umsatz.setBetrag(Utils.convertStringToDouble(transaction[3]));

		// Auszug
		if (StringUtils.isNotEmpty(transaction[4])) {
			umsatz.setKommentar(I18N.tr("BceeWebSiteTransactionsImpl.accountSheet") + transaction[4]); //$NON-NLS-1$
		}

		// Saldo
		if (StringUtils.isNotEmpty(transaction[5])) {
			umsatz.setSaldo(Utils.convertStringToDouble(transaction[5]));
		}

		// Transaktion
		umsatz.setArt(transaction[6]);

		return umsatz;
	}

	/**
	 * Lscht die interen Umsatzliste
	 */
	public void clear() {
		this.allTransactions.clear();
	}
}
