//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.assistant;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateKontoDialog;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateAccountInfo;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Account;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.input.SelectInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.hbci.gui.input.BICInput;

/**
 * Dialog zum Abfragen der MahoeMain Kreditkarteninformationen
 * 
 * @author Frank Mnster
 * 
 */
public class BshCreateAccountDialog extends AbstractCreateKontoDialog<BshCreateInfo> {

	/**
	 * Liste der vorhandenen Vertrge
	 */
	private final transient List<Account> accounts;

	/**
	 * Select Input fr die Accounts
	 */
	private transient SelectInput accountInput;

	/**
	 * Ausgewhlter Account
	 */
	// private final transient Account choosen;

	/**
	 * Kundennummer
	 */
	private final transient String kdNummer;

	/**
	 * Kundennummer Input
	 */
	private transient TextInput kdNummerInput;

	/**
	 * Erstellt den BSH Konto-Anlage Dialog
	 * 
	 * @param kdNummer
	 * @param accounts
	 */
	public BshCreateAccountDialog(final String kdNummer, final List<Account> accounts) {
		super();
		this.setTitle(I18N.tr("BshCreateAccountDialog.dialogTitle")); //$NON-NLS-1$
		this.kdNummer = kdNummer;
		this.accounts = accounts;
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void paint(final Composite parent) {

		super.paint(parent);

		kdNummerInput = new TextInput(kdNummer, 20, StringUtils.EMPTY);
		kdNummerInput.setEnabled(false);
		final BICInput bic = new BICInput(SchwaebischHall.BIC); // $NON-NLS-1$
		bic.setEnabled(false);

		container.addLabelPair(I18N.tr("BshCreateAccountDialog.kdNumber"), kdNummerInput); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("BshCreateAccountDialog.bic"), bic); //$NON-NLS-1$

		accountInput = new SelectInput(accounts, accounts.get(0));
		accountInput.setAttribute("bsnType"); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("BshCreateAccountDialog.existingAccounts"), accountInput); //$NON-NLS-1$

	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
	 */
	@Override
	protected BshCreateInfo getData() {

		final CreateAccountInfo info = super.getData();

		data.setOwner(info.getOwner());
		data.setComment(info.getComment());
		data.setDescription(StringUtils.isEmpty(info.getDescription()) ? "Schwbisch Hall" //$NON-NLS-1$
				: info.getDescription());

		return data;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateKontoDialog#prepareData()
	 */
	@Override
	protected boolean prepareData() {
		data = new BshCreateInfo();

		data.setKdNummer((String) kdNummerInput.getValue());
		final Account choosen = (Account) accountInput.getValue();
		data.setAccount(choosen);

		// TODO DatenCheck einbauen
		return true;

	}

}
