//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json;

import java.math.BigDecimal;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.MoneyDeserializer;

/**
 * JSON Account Object (ein Vertrag)
 * 
 * @author Frank Mnster/Quirin Huber
 * 
 */
@JsonIgnoreProperties(ignoreUnknown = true)
public class Account {

	/**
	 * Laufendes Saldo
	 */
	@JsonDeserialize(using = MoneyDeserializer.class)
	private transient BigDecimal accountBalance;

	/**
	 * Die Bausparnummer
	 */
	@JsonProperty
	private transient String bsn;

	/**
	 * Der Vertragsbuchstabe
	 */
	@JsonProperty("contractletter")
	private transient String contractLetter;

	/**
	 * Die Vertrags-Suffixnummer
	 */
	@JsonProperty("contractnumber")
	private transient String contractNumber;

	/**
	 * Die Vertragsnummer
	 */
	@JsonProperty("coreid")
	private transient String coreId;

	/**
	 * Der Vertagstyp
	 */
	@JsonProperty
	private transient String type;

	/**
	 * Die BVTN
	 */
	@JsonProperty
	private transient String vtn;

	// Wozu das wohl mal gut sein soll
	// @JsonProperty
	// private Map<String, String> visual;

	/**
	 * Standardkonstruktor
	 */
	public Account() {
		// Absichtlich leer
	}

	/**
	 * Liefert das laufende Saldo
	 * 
	 * @return das laufende Saldo
	 */
	public BigDecimal getAccountBalance() {
		return accountBalance;
	}

	/**
	 * Liefert die BSN
	 * 
	 * @return die BSN
	 */
	public String getBsn() {
		return bsn;
	}

	/**
	 * Liefert die BSN und den Typen
	 * 
	 * @return die BSN und den Typen
	 */
	public String getBsnType() {
		return bsn + " - " + type; //$NON-NLS-1$
	}

	/**
	 * Liefert den Vertagsbuchstaben
	 * 
	 * @return den Vertagsbuchstaben
	 */
	public String getContractLetter() {
		return contractLetter;
	}

	/**
	 * Liefert die Vertrags-Suffixnummer
	 * 
	 * @return die Vertrags-Suffixnummer
	 */
	public String getContractNumber() {
		return contractNumber;
	}

	/**
	 * Liefert die Vertragshauptnummer
	 * 
	 * @return die Vertragshauptnummer
	 */
	public String getCoreId() {
		return coreId;
	}

	/**
	 * Liefert den Vertragstyp
	 * 
	 * @return den Vertragstyp
	 */
	public String getType() {
		return type;
	}

	/**
	 * Liefert die VTN
	 * 
	 * @return die VTN
	 */
	public String getVtn() {
		return vtn;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return String.format(
				"Account [accountBalance=%s, bsn=%s, contractLetter=%s, contractNumber=%s, coreId=%s, type=%s, vtn=%s]", //$NON-NLS-1$
				accountBalance, bsn, contractLetter, contractNumber, coreId, type, vtn);
	}
}
