//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;

/**
 * Hilfsklasse die eine Umsatzliste aus den Vertragsdetails erstellt.
 * 
 * @author Frank Mnster
 * 
 */
public final class AccountDetailMapper {

	/**
	 * Die Vertragdetails mit dem Umstzen der letzten 3 Jahre
	 */
	private AccountDetailData data;

	/**
	 * Erzeugt einen AccountMapper fr die AccountDate
	 * 
	 * @param data
	 */
	public AccountDetailMapper(final AccountDetailData data) {
		this.data = data;
	}

	/**
	 * @return the data
	 */
	public AccountDetailData getData() {
		return data;
	}

	/**
	 * @param data
	 *            the data to set
	 */
	public void setData(final AccountDetailData data) {
		this.data = data;
	}

	/**
	 * Liefert die Umsatzliste aus den Vertragdetails
	 * 
	 * @return die Umsatzliste
	 */
	public List<Umsatz> getUmsatzList() {

		final List<Umsatz> umsaetze = new ArrayList<Umsatz>();

		for (final TransactionYear transactionYear : data.getTransactions()) {

			// final int year = transactionYear.getYear();
			BigDecimal saldo = transactionYear.getSaldo();

			for (final TransactionMonth transactionMonth : transactionYear.getMonth()) {

				// final int month = transactionMonth.getMonthAsInt();
				for (final TransactionEntry transactionEntry : transactionMonth.getEntries()) {

					// final Date entryDate = getEntryDate(transactionEntry,
					// month, year);
					addEntry(umsaetze, transactionEntry, saldo);
					if (transactionEntry.getAmount() != null) {
						saldo = saldo.subtract(transactionEntry.getAmount());
					}

				}

			}

		}

		return umsaetze;

	}

	private void addEntry(final List<Umsatz> umsaetze, final TransactionEntry transactionEntry,
			final BigDecimal saldo) {

		try {
			// Neuen Umsatz anlegen
			final HBCIDBService dbService = Settings.getDBService();
			final Umsatz umsatz = dbService.createObject(Umsatz.class, null);

			// Datum
			umsatz.setDatum(transactionEntry.getDate());
			umsatz.setValuta(transactionEntry.getDate());

			// Zweck
			Utils.loadUmsatz(umsatz, transactionEntry.getSubject());

			// Betrag
			umsatz.setBetrag(transactionEntry.getAmount().doubleValue());

			// Saldo
			umsatz.setSaldo(saldo.doubleValue());

			// Zur Liste hinzufgen
			umsaetze.add(umsatz);

		} catch (RemoteException re) {
			Logger.error(I18N.tr("AccountDetailMapper.createUmsatzFailed", re.getMessage()), re); //$NON-NLS-1$
		}

	}

}
