//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.SimpleMoneyDeserializer;

/**
 * Vertragsdetails - Directdebit (Einzug)
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "owner", "iban", "bic", "bank", "interval", "amount", "startdate", "enddate" })
public class DirectDebit {

	/**
	 * Einzugsbetrag
	 */
	@JsonProperty("amount")
	@JsonDeserialize(using = SimpleMoneyDeserializer.class)
	private transient BigDecimal amount;

	/**
	 * Die Bank
	 */
	@JsonProperty("bank")
	private transient String bank;

	/**
	 * BIC der Bank
	 */
	@JsonProperty("bic")
	private transient String bic;

	/**
	 * IBAN der Bank
	 */
	@JsonProperty("iban")
	private transient String iban;

	/**
	 * Einzugsturnus
	 */
	@JsonProperty("interval")
	private transient String interval;

	/**
	 * Kontoinhaber
	 */
	@JsonProperty("owner")
	private transient String owner;

	/**
	 * Anfangsdatum
	 */
	@JsonProperty("startdate")
	@JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "dd.MM.yyyy", timezone = "CET")
	private transient Date startDate;

	/**
	 * Enddatum
	 */
	@JsonProperty("enddate")
	@JsonFormat(shape = JsonFormat.Shape.STRING, pattern = "dd.MM.yyyy", timezone = "CET")
	private transient Date endDate;

	/**
	 * Standardkonstruktor
	 */
	public DirectDebit() {
		// Absichtlich leer
	}

	/**
	 * Liefert den Einzugsbetrag
	 * 
	 * @return den Einzugsbetrag
	 */
	@JsonProperty("amount")
	public BigDecimal getAmount() {
		return this.amount;
	}

	/**
	 * Liefert die Bank
	 * 
	 * @return die Bank
	 */
	@JsonProperty("bank")
	public String getBank() {
		return this.bank;
	}

	/**
	 * Liefert die BIC
	 * 
	 * @return die BIC
	 */
	@JsonProperty("bic")
	public String getBic() {
		return this.bic;
	}

	/**
	 * Liefert die IBAN
	 * 
	 * @return die IBAN
	 */
	@JsonProperty("iban")
	public String getIban() {
		return this.iban;
	}

	/**
	 * Liefert den Einzugsturnus
	 * 
	 * @return den Einzugsturnus
	 */
	@JsonProperty("interval")
	public String getInterval() {
		return this.interval;
	}

	/**
	 * Liefert den Kontoinhaber
	 * 
	 * @return den Kontoinhaber
	 */
	@JsonProperty("owner")
	public String getOwner() {
		return this.owner;
	}

	/**
	 * Liefert das Startdatum
	 * 
	 * @return das Startdatum
	 */
	@JsonProperty("startdate")
	public Date getStartDate() {
		return this.startDate;
	}

	/**
	 * Liefert das Enddatum
	 * 
	 * @return das Enddatum
	 */
	@JsonProperty("enddate")
	public Date getEndDate() {
		return this.endDate;
	}

}
