//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;

import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.MoneyDeserializer;
import de.frankmuenster.hibiscus.mahoe.bsh.json.deserializer.TransactionDateDeserializer;

/**
 * Vertargdetails - Umsatzeintrag
 * 
 * @author Frank Mnster
 * 
 */
@JsonInclude(JsonInclude.Include.NON_NULL)
@JsonPropertyOrder({ "amount", "subject", "day" })
public class TransactionEntry {

	/**
	 * Der Buchungsbetrag
	 */
	@JsonProperty("amount")
	@JsonDeserialize(using = MoneyDeserializer.class)
	private transient BigDecimal amount;

	/**
	 * Der Buchungstext
	 */
	@JsonProperty("subject")
	private transient String subject;

	/**
	 * Das Buchungsdatum
	 */
	@JsonProperty("day")
	@JsonDeserialize(using = TransactionDateDeserializer.class)
	private transient Date date;

	/**
	 * Standardkonstruktor
	 */
	public TransactionEntry() {
		// Absichtlich leer
	}

	/**
	 * Liefert den Betrag der Buchung.
	 * 
	 * @return den Betrag der Buchung
	 */
	@JsonProperty("amount")
	public BigDecimal getAmount() {
		return amount;
	}

	/**
	 * Liefert den Buchungstext.
	 * 
	 * @return der Buchungstext
	 */
	@JsonProperty("subject")
	public String getSubject() {
		return subject;
	}

	/**
	 * Der Tag der Buchung
	 * 
	 * @return der Buchungstag
	 */
	public Integer getDay() {
		final Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		return cal.get(Calendar.DATE);
	}

	/**
	 * Das Datum der Buchung
	 * 
	 * @return das Datum der Buchung
	 */
	@JsonProperty("day")
	public Date getDate() {
		return date;
	}

}
