//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.bsh.web;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpHeaders;
import org.apache.http.entity.ContentType;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.HttpMethod;
import com.gargoylesoftware.htmlunit.NicelyResynchronizingAjaxController;
import com.gargoylesoftware.htmlunit.Page;
import com.gargoylesoftware.htmlunit.UnexpectedPage;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.WebClientOptions;
import com.gargoylesoftware.htmlunit.WebRequest;
import com.gargoylesoftware.htmlunit.WebResponse;
import com.gargoylesoftware.htmlunit.util.NameValuePair;

import de.frankmuenster.hibiscus.mahoe.bcee.Bcee;
import de.frankmuenster.hibiscus.mahoe.bsh.SchwaebischHall;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Account;
import de.frankmuenster.hibiscus.mahoe.bsh.json.AccountDetailsResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.AccountListResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Error;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Exemption;
import de.frankmuenster.hibiscus.mahoe.bsh.json.ExemptionsResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.IbanResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Message;
import de.frankmuenster.hibiscus.mahoe.bsh.json.MessageCountResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.MessageReadResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.MessagesResponse;
import de.frankmuenster.hibiscus.mahoe.bsh.json.Status;
import de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails.AccountDetailData;
import de.frankmuenster.hibiscus.mahoe.bsh.json.accountdetails.AccountDetailMapper;
import de.frankmuenster.hibiscus.mahoe.messages.MahoeMessage;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;

/**
 * Utility-Klasse fr die BSH Website
 * 
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.GodClass" })
public final class BshWebSiteUtils {

	/**
	 * Jackson Mapper
	 */
	private static final ObjectMapper MAPPER = new ObjectMapper();

	/**
	 * MessageRead ID POST-Parameter
	 */
	private static final String PARAM_ID = "id"; //$NON-NLS-1$
	/**
	 * MessageRead Command POST-Parameter
	 */
	private static final String PARAM_COMMAND = "command"; //$NON-NLS-1$
	/**
	 * MessageRead Command
	 */
	private static final String COMMAND_READ = "readmessage"; //$NON-NLS-1$

	/**
	 * Privater Konstruktor
	 */
	private BshWebSiteUtils() {
		throw new AssertionError("Instantiating utility class..."); //$NON-NLS-1$
	}

	/**
	 * Liefert einen instanzierten HtmlUnit WebClient
	 * 
	 * @return der WebClient
	 */
	/* default */static WebClient getWebClient() { // NOPMD

		final WebClient webcl = new WebClient();

		final WebClientOptions options = webcl.getOptions();

		options.setUseInsecureSSL(false);
		options.setJavaScriptEnabled(true);
		options.setThrowExceptionOnFailingStatusCode(false);
		options.setCssEnabled(true);
		options.setThrowExceptionOnScriptError(false);
		options.setRedirectEnabled(true);

		webcl.setAjaxController(new NicelyResynchronizingAjaxController());
		webcl.getCookieManager().setCookiesEnabled(true);

		Utils.setProxyCfg(webcl, Bcee.URL_LOGIN);

		return webcl;

	}

	/**
	 * berprft die Webantwort, ob der http Statuscode 200 ist und die Antwort
	 * im JSON-Format vorliegt.
	 * 
	 * @param webResponse
	 * @return true wenn HHTP code 200 und Antwortinhalt application/json ist,
	 *         sonst false.
	 */
	/* default */static boolean webResponseIsJson(final WebResponse webResponse) { // NOPMD

		boolean answerOK = false;
		if (webResponse.getStatusCode() == org.apache.http.HttpStatus.SC_OK
				&& ContentType.APPLICATION_JSON.getMimeType().equalsIgnoreCase(webResponse.getContentType())) { // $NON-NLS-1$
			answerOK = true;

		} else {
			// Ein Debuginfos loggen
			Logger.debug("JSON-Check-Status: " + webResponse.getStatusCode()); //$NON-NLS-1$
			Logger.debug("JSON-Check-MimeType: " + webResponse.getContentType()); //$NON-NLS-1$
			Logger.debug("JSON-Check-Inhalt: " + webResponse.getContentAsString()); //$NON-NLS-1$
		}
		return answerOK;
	}

	/* default */static List<Account> getAccounts(final WebClient webClient) throws WebSiteException { // NOPMD

		List<Account> accounts = new ArrayList<>();
		try {
			final Page responsePage = webClient.getPage(SchwaebischHall.URL_CONTRACTS);
			if (webResponseIsJson(responsePage.getWebResponse())) {

				final UnexpectedPage page = (UnexpectedPage) responsePage;
				Logger.debug("GetAccounts WebResponseCode :" + page.getWebResponse().getStatusCode()); //$NON-NLS-1$
				Logger.debug(responsePage.getWebResponse().getContentAsString());

				final AccountListResponse accountResponse = MAPPER
						.readValue(responsePage.getWebResponse().getContentAsStream(), AccountListResponse.class);
				if (accountResponse.getStatus() == Status.ERROR) {
					final List<Error> errors = accountResponse.getErrors();
					for (final Error error : errors) {
						Logger.error(error.getMessage());
						throw new WebSiteException(accountResponse.getErrors().get(0).getMessage(),
								WebSiteExceptionTypes.LOGIN_FAILED);
					}
				}

				accounts = accountResponse.getAccounts();

			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(e.getMessage());
			throw new WebSiteException(I18N.tr("BshWebSiteUtils.accountRequestFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
		}

		return accounts;
	}

	/**
	 * Liefert die IBAN
	 * 
	 * @param webClient
	 * @param bsnKurz
	 * @param blz
	 * @return die IBAN, oder NULL
	 * @throws WebSiteException
	 */
	/* default */static String getIban(final WebClient webClient, final String bsnKurz, final String blz) // NOPMD
			throws WebSiteException {

		String iban = null;
		try {
			final String url = SchwaebischHall.URL_IBAN + "?bsn=" + bsnKurz + "&blz=" + blz; //$NON-NLS-1$ //$NON-NLS-2$
			final Page responsePage = webClient.getPage(url);
			if (webResponseIsJson(responsePage.getWebResponse())) {

				final UnexpectedPage pape = (UnexpectedPage) responsePage;
				Logger.debug("GetIban WebResponseCode :" + pape.getWebResponse().getStatusCode()); //$NON-NLS-1$
				Logger.debug(responsePage.getWebResponse().getContentAsString());

				final IbanResponse ibanResponse = MAPPER.readValue(responsePage.getWebResponse().getContentAsStream(),
						IbanResponse.class);
				if (ibanResponse.getStatus() == Status.ERROR) {
					final List<Error> errors = ibanResponse.getErrors();
					for (final Error error : errors) {
						Logger.error(error.getMessage());
						throw new WebSiteException(ibanResponse.getErrors().get(0).getMessage(),
								WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED);
					}
				}

				iban = ibanResponse.getIban();

			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(e.getMessage());
			throw new WebSiteException(I18N.tr("BshWebSiteUtils.ibanRequestFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
		}

		return iban;
	}

	/**
	 * Liefert die allg. Vertragsdaten zu den gegebenen VTN.
	 * 
	 * @param webClient
	 * @param vtn
	 * @return die allg. Vertragsdaten.
	 * 
	 * @throws WebSiteException
	 *             , falls kein Vertrag mit der angegebenen VTN gefunden wurde.
	 */
	public static Account getAccount(final WebClient webClient, final String vtn) throws WebSiteException {

		final List<Account> accounts = getAccounts(webClient);
		Account account = null;
		for (final Account acc : accounts) {
			if (acc.getVtn().equals(vtn)) {
				account = acc;
				break;
			}
		}
		if (account == null) {
			throw new WebSiteException(I18N.tr("BshWebSiteUtils.contractNotExists", vtn), //$NON-NLS-1$
					WebSiteExceptionTypes.KONTO_NOT_FOUND);
		}
		return account;
	}

	/**
	 * Liefert das AccountDetailObject mit allen Angaben zu dem Vertrag.
	 * 
	 * @param webClient
	 * @param vtn
	 *            die VTN Nummer des Vertrages.
	 * 
	 * @return das AccountDetailObject
	 * @throws WebSiteException
	 */
	public static AccountDetailData getAccountDetails(final WebClient webClient, final String vtn)
			throws WebSiteException {

		AccountDetailData accountDetails = null;
		try {
			final String url = SchwaebischHall.URL_DETAILS + "?vtn=" + vtn; //$NON-NLS-1$
			final Page responsePage = webClient.getPage(url);
			if (webResponseIsJson(responsePage.getWebResponse())) {

				final UnexpectedPage pape = (UnexpectedPage) responsePage;
				Logger.debug("GetAccountDetails RWebResponseCode :" + pape.getWebResponse().getStatusCode()); //$NON-NLS-1$
				Logger.debug(responsePage.getWebResponse().getContentAsString());

				final AccountDetailsResponse deatilsResponse = MAPPER
						.readValue(responsePage.getWebResponse().getContentAsStream(), AccountDetailsResponse.class);
				if (deatilsResponse.getStatus() == Status.ERROR) {
					final List<Error> errors = deatilsResponse.getErrors();
					for (final Error error : errors) {
						Logger.error(error.getMessage());
						throw new WebSiteException(deatilsResponse.getErrors().get(0).getMessage(),
								WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED);
					}
				}

				accountDetails = deatilsResponse.getAccountDetails();

			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(e.getMessage());
			throw new WebSiteException(I18N.tr("BshWebSiteUtils.accountDetailsRequestFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
		}

		return accountDetails;
	}

	/**
	 * Liefert eine Liste mit Ums&auml;tzen aus den Vertragsdetails-Objekt.
	 * 
	 * @param accountDetailData
	 *            das Vertragsdetails-Objekt
	 * @return Liste mit Hibiscus-Ums&auml;tzen
	 * 
	 * @throws WebSiteException
	 */
	public static List<Umsatz> getBookedTransctions(final AccountDetailData accountDetailData) throws WebSiteException { // NOPMD

		List<Umsatz> umsaetze = null;
		final AccountDetailMapper mapper = new AccountDetailMapper(accountDetailData);
		umsaetze = mapper.getUmsatzList();

		return umsaetze;
	}

	/**
	 * Liefert die Anzahl neuer ungelesener Nachrichten.
	 * 
	 * @param webClient
	 *            der angemeldete WebClient
	 * @return die Anzahl neuer ungelesener Nachrichten
	 * @throws WebSiteException
	 */
	public static int getNewMessageCount(final WebClient webClient) throws WebSiteException { // NOPMD

		int msgCount = 0;

		try {
			final String url = SchwaebischHall.URL_MESSAGE_COUNT;
			final Page responsePage = webClient.getPage(url);
			if (webResponseIsJson(responsePage.getWebResponse())) {

				final UnexpectedPage pape = (UnexpectedPage) responsePage;
				Logger.debug("GetNewMessageCount WebResponseCode :" + pape.getWebResponse().getStatusCode()); //$NON-NLS-1$
				Logger.debug(responsePage.getWebResponse().getContentAsString());

				final MessageCountResponse deatilsResponse = MAPPER
						.readValue(responsePage.getWebResponse().getContentAsStream(), MessageCountResponse.class);
				if (deatilsResponse.getStatus() == Status.ERROR) {
					final List<Error> errors = deatilsResponse.getErrors();
					for (final Error error : errors) {
						Logger.error(error.getMessage());
						throw new WebSiteException(deatilsResponse.getErrors().get(0).getMessage(),
								WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED);
					}
				}

				msgCount = deatilsResponse.getMsgCount().getCount();

			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(e.getMessage());
			throw new WebSiteException(I18N.tr("BshWebSiteUtils.getNewMeassagesRequestfailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
		}

		return msgCount;
	}

	/**
	 * Liefert die Anzahl neuer ungelesener Nachrichten.
	 * 
	 * @param webClient
	 *            der angemeldete WebClient
	 * @return die Anzahl neuer ungelesener Nachrichten
	 * @throws WebSiteException
	 */
	public static List<MahoeMessage> getNewMessages(final WebClient webClient) throws WebSiteException { // NOPMD

		final List<MahoeMessage> messages = new ArrayList<>();

		try {
			final String url = SchwaebischHall.URL_MESSAGES;
			final Page responsePage = webClient.getPage(url);
			if (webResponseIsJson(responsePage.getWebResponse())) {

				final UnexpectedPage pape = (UnexpectedPage) responsePage;
				Logger.debug("GetNewMessages WebResponseCode :" + pape.getWebResponse().getStatusCode()); //$NON-NLS-1$
				Logger.debug(responsePage.getWebResponse().getContentAsString());

				final MessagesResponse deatilsResponse = MAPPER
						.readValue(responsePage.getWebResponse().getContentAsStream(), MessagesResponse.class);
				if (deatilsResponse.getStatus() == Status.ERROR) {
					final List<Error> errors = deatilsResponse.getErrors();
					for (final Error error : errors) {
						Logger.error(error.getMessage());
						throw new WebSiteException(deatilsResponse.getErrors().get(0).getMessage(),
								WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED);
					}
				}

				final List<Message> msgs = deatilsResponse.getMessages();
				for (final Message msg : msgs) {
					if (!msg.isRead()) {
						String text = msg.getText().replaceAll("\n", StringUtils.EMPTY); //$NON-NLS-1$
						text = text.replaceAll("<br />", "\n"); //$NON-NLS-1$ //$NON-NLS-2$
						if (msg.getType().equals("data")) { //$NON-NLS-1$ //
															// NOPMD
							final StringBuilder builder = new StringBuilder(text.length() + 106); // NOPMD
							builder.append(I18N.tr("BshWebSiteUtils.attachmentWarning1") //$NON-NLS-1$
									+ I18N.tr("BshWebSiteUtils.attachmentWarning2")) //$NON-NLS-1$
									.append(text);
							text = builder.toString();
						}

						final MahoeMessage message = new MahoeMessage(SchwaebischHall.BLZ, msg.getDate(), // NOPMD
								msg.getTitle(), null);
						message.setMessage(text);
						message.setId(msg.getId());
						messages.add(message);
					}
				}

			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(e.getMessage());
			throw new WebSiteException(I18N.tr("BshWebSiteUtils.getMeassagesRequestFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
		}

		return messages;
	}

	/**
	 * Markiert die Nachricht als gelesen.
	 * 
	 * @param webClient
	 * 
	 * @param message
	 * @throws WebSiteException
	 */
	public static void markMessageAsRead(final WebClient webClient, final MahoeMessage message)
			throws WebSiteException {
		try {

			final URL url = new URL(SchwaebischHall.URL_POSTBOX);
			final WebRequest requestSetting = new WebRequest(url, HttpMethod.POST);
			requestSetting.setAdditionalHeader(HttpHeaders.USER_AGENT, BshWebSite.CUSTOM_USER_AGENT);
			final List<NameValuePair> urlParameters = new ArrayList<>();
			urlParameters.add(new NameValuePair(PARAM_ID, message.getId()));
			urlParameters.add(new NameValuePair(PARAM_COMMAND, COMMAND_READ));
			requestSetting.setRequestParameters(urlParameters);

			final Page responsePage = webClient.getPage(requestSetting);
			if (webResponseIsJson(responsePage.getWebResponse())) {

				final UnexpectedPage pape = (UnexpectedPage) responsePage;
				Logger.debug("GetNewMessages WebResponseCode :" + pape.getWebResponse().getStatusCode()); //$NON-NLS-1$
				Logger.debug(responsePage.getWebResponse().getContentAsString());

				final MessageReadResponse deatilsResponse = MAPPER
						.readValue(responsePage.getWebResponse().getContentAsStream(), MessageReadResponse.class);
				if (deatilsResponse.getStatus() == Status.ERROR) {
					final List<Error> errors = deatilsResponse.getErrors();
					for (final Error error : errors) {
						Logger.error(error.getMessage());
						throw new WebSiteException(deatilsResponse.getErrors().get(0).getMessage(),
								WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED);
					}
				}

			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(e.getMessage());
			throw new WebSiteException(I18N.tr("BshWebSiteUtils.markReadRequestFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
		}
	}

	/**
	 * Liefert eine Liste mit den Freistellungsantr&auml;gen.
	 * 
	 * @param webClient
	 *            das angemeldete WebClient Objekt.
	 * 
	 * @return eine Liste mit den Freistellungsantr&auml;gen.
	 * @throws WebSiteException
	 */
	public static List<Exemption> getExemptions(final WebClient webClient) throws WebSiteException { // NOPMD

		List<Exemption> exemptions = new ArrayList<>();
		try {
			final Page responsePage = webClient.getPage(SchwaebischHall.URL_EXEMPTION);
			if (webResponseIsJson(responsePage.getWebResponse())) {

				final UnexpectedPage page = (UnexpectedPage) responsePage;
				Logger.debug("GetAccounts WebResponseCode :" + page.getWebResponse().getStatusCode()); //$NON-NLS-1$
				Logger.debug(responsePage.getWebResponse().getContentAsString());

				final ExemptionsResponse exemptionResponse = MAPPER
						.readValue(responsePage.getWebResponse().getContentAsStream(), ExemptionsResponse.class);
				if (exemptionResponse.getStatus() == Status.ERROR) {
					final List<Error> errors = exemptionResponse.getErrors();
					for (final Error error : errors) {
						Logger.error(error.getMessage());
						throw new WebSiteException(exemptionResponse.getErrors().get(0).getMessage(),
								WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED);
					}
				}
				exemptions = exemptionResponse.getExemptions();
			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(e.getMessage());
			throw new WebSiteException(I18N.tr("BshWebSiteUtils.accountRequestFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
		}

		return exemptions;
	}

	/**
	 * Liefert den Freistellungsauftrag zur CoreID.
	 * 
	 * @param webClient
	 * @param coreId
	 * 
	 * @return den Freistellungsauftrag, oder <i>NULL</i>
	 * @throws WebSiteException
	 */
	public static Exemption getExemption(final WebClient webClient, final String coreId) throws WebSiteException {

		Exemption result = null;
		final List<Exemption> exemptions = getExemptions(webClient);
		for (final Exemption exemption : exemptions) {
			if (coreId.equals(exemption.getCoreid())) {
				result = exemption;
				break;
			}
		}
		return result;
	}

}
