//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.santander.assistant;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.assistant.AbstractAccountMigrator;
import de.frankmuenster.hibiscus.mahoe.assistant.InterfaceMahoeAccountMigrator;
import de.frankmuenster.hibiscus.mahoe.santander.SantanderBank;
import de.frankmuenster.hibiscus.mahoe.santander.web.SantanderBankEnum;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.SynchronizeOptions;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.plugin.Plugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Der Migrator fr existierende BOS Konten
 * 
 * @author Frank Mnster
 * 
 */

public class SantanderAccountMigrator extends AbstractAccountMigrator implements InterfaceMahoeAccountMigrator {

	/**
	 * Der alte Plugin Name
	 */
	private final static String OLD_PLUGIN_NAME = "hibiscus.script.santander_consumer"; //$NON-NLS-1$

	/**
	 * Standardkonstruktor
	 */
	public SantanderAccountMigrator() {
		super();
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.assistant.AbstractAccountMigrator#migrate
	 *      (de.willuhn.jameica.plugin.Plugin)
	 */
	@Override
	public void migrate(final Plugin plugin) throws ApplicationException {

		int countMigrated = 0;

		// Da wir kein altes Plugin haben, mach wir es etwas anders
		if (OLD_PLUGIN_NAME.equals(plugin.getManifest().getName())) {

			Logger.info("Santander-Migrator gestartet."); //$NON-NLS-1$
			try {
				final HBCIDBService hcbiService = de.willuhn.jameica.hbci.Settings.getDBService();
				final DBIterator<Konto> accounts = hcbiService.createList(Konto.class);
				while (accounts.hasNext()) {
					final Konto account = accounts.next();
					if (SantanderBank.BLZ.equals(account.getBLZ())
							&& !MahoeSynchronizeBackend.BACKEND_NAME.equals(account.getBackendClass())) {
						migrateAccount(account);
						countMigrated++;
					}
				}

			} catch (RemoteException e) {
				Logger.error(I18N.tr("SantanderAccountMigrator.readAccountsFailed", e.getMessage()), e); //$NON-NLS-1$
			}

			if (countMigrated > 0) {
				final StatusBarMessage statusBarMsg = new StatusBarMessage(
						I18N.tr("SantanderAccountMigrator.accountMigratedSuccessfully"), StatusBarMessage.TYPE_SUCCESS); //$NON-NLS-1$
				Application.getMessagingFactory().sendMessage(statusBarMsg);
			}

			// Plugin entfernen
			uninstallPlugin(plugin);

		}

	}

	/**
	 * Wir mssen nur die Backend-Engine ndern.
	 * 
	 * @param konto
	 *            das zu migrierende Konto
	 * 
	 * @throws ApplicationException
	 */
	private void migrateAccount(final Konto konto) throws ApplicationException {

		try {

			// Kreditkarten ausklammern
			final KontoType kontoType = KontoType.find(konto.getAccountType());
			if (!KontoType.KREDITKARTE.equals(kontoType)) {
				final String unterkonto = konto.getUnterkonto().toLowerCase();

				konto.setUnterkonto(StringUtils.EMPTY);
				konto.setBackendClass("de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend"); //$NON-NLS-1$
				konto.setAccountType(KontoType.GIRO.getValue());
				konto.store();

				// Meta Daten
				final String bank = SantanderBankEnum.DIREKT.getIdString().equals(unterkonto)
						? SantanderBankEnum.DIREKT.getIdString() : SantanderBankEnum.CONSUMER.getIdString();

				konto.setMeta(MahoeSynchronizeBackend.BANK_NAME, bank);
				konto.setMeta(MahoeSynchronizeBackend.KONTO_VERSION, SantanderBank.KTO_VER_GIRO);
				konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF, "NEIN"); //$NON-NLS-1$
				konto.setMeta(MahoeSynchronizeBackend.SAVE_PDF_PATH, StringUtils.EMPTY);

				// Sync-Options setzen, nur zur Sicherheit
				final SynchronizeOptions sync = new SynchronizeOptions(konto);
				sync.setAutoSaldo(false);
				sync.setSyncAuslandsUeberweisungen(false);
				sync.setSyncKontoauszuege(true);
				sync.setSyncOffline(false);
				sync.setSyncSaldo(true);
				sync.setSyncSepaDauerauftraege(false);
				sync.setSyncSepaLastschriften(false);
			}

		} catch (RemoteException e) {
			Logger.error(I18N.tr("SantanderAccountMigrator.accountMigrationFailedLog", e.getMessage()), e); //$NON-NLS-1$
			throw new ApplicationException(I18N.tr("SantanderAccountMigrator.accountMigrationFailed", e.getMessage()), //$NON-NLS-1$
					e);
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.assistant.AbstractAccountMigrator#update(de.willuhn.jameica.hbci.rmi.Konto)
	 */
	@Override
	public void update(final Konto account) throws ApplicationException {
		// TODO Auto-generated method stub

	}

}
