//
// Copyright 2017 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.santander.assistant;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateKontoDialog;
import de.frankmuenster.hibiscus.mahoe.assistant.CreateAccountInfo;
import de.frankmuenster.hibiscus.mahoe.santander.SantanderBank;
import de.frankmuenster.hibiscus.mahoe.santander.web.SantanderBankEnum;
import de.frankmuenster.hibiscus.mahoe.utils.DebitCardValidator;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.DirectoryInput;
import de.willuhn.jameica.gui.input.SelectInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.hbci.HBCIProperties;
import de.willuhn.jameica.hbci.rmi.KontoType;

/**
 * Kontoanlage Dialog der Santander Bank
 * 
 * @author Frank Mnster
 * 
 */
public class SantanderCreateAccountDialog extends AbstractCreateKontoDialog<SantanderCreateInfo> {

	/**
	 * Benutername Input
	 */
	private transient TextInput login;

	/**
	 * Kontonummer Input
	 */
	private transient TextInput ktoNummer;

	/**
	 * Auszge speichern Input
	 */
	private transient CheckboxInput savePDF;

	/**
	 * Speicherordner Input
	 */
	private transient DirectoryInput saveFolder;

	/**
	 * Banktyp
	 */
	private transient SelectInput bankTypeInput;

	/**
	 * Kontotyp
	 */
	private transient SelectInput ktoTypeInput;

	/**
	 */
	public SantanderCreateAccountDialog() {
		super();
		this.setTitle(I18N.tr("SantanderCreateAccountDialog.dialogTitle")); //$NON-NLS-1$
	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected void paint(final Composite parent) {

		super.paint(parent);

		final List<SantanderBankEnum> santanderBanks = Arrays.asList(SantanderBankEnum.values());
		bankTypeInput = new SelectInput(santanderBanks, santanderBanks.get(0));
		bankTypeInput.setAttribute("bankName"); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.existingBanks"), bankTypeInput); //$NON-NLS-1$

		final List<String> ktoTypen = new ArrayList<String>();
		ktoTypen.add(I18N.tr("SantanderCreateAccountDialog.instantAccessSavingsAccount")); //$NON-NLS-1$
		ktoTypen.add(I18N.tr("SantanderCreateAccountDialog.creditcard")); //$NON-NLS-1$
		ktoTypeInput = new SelectInput(ktoTypen, ktoTypen.get(0));
		container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.kinfOfAccount"), ktoTypeInput); //$NON-NLS-1$

		login = new TextInput(StringUtils.EMPTY, 20, StringUtils.EMPTY);
		login.setComment(StringUtils.EMPTY);
		login.setMandatory(true);
		ktoNummer = new TextInput(StringUtils.EMPTY, 20, StringUtils.EMPTY);
		ktoNummer.setComment(I18N.tr("SantanderCreateAccountDialog.differentAsLogin")); //$NON-NLS-1$
		accDescription.setHint("Santander Bank"); //$NON-NLS-1$

		savePDF = new CheckboxInput(false);
		saveFolder = new DirectoryInput(StringUtils.EMPTY);

		container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.login"), login); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.ktoNumber"), ktoNummer); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.lblsavePdf"), savePDF); //$NON-NLS-1$
		container.addLabelPair(I18N.tr("SantanderCreateAccountDialog.lblsavePdfFolder"), saveFolder); //$NON-NLS-1$

		container.addText(" ", true); //$NON-NLS-1$
		final StringBuffer formString = new StringBuffer(300) // NOPMD 
				.append("<form><p>") //$NON-NLS-1$ // NOPMD 
				.append(I18N.tr("SantanderCreateAccountDialog.websiteHint")) //$NON-NLS-1$
				.append("<br/><a href=\"http://progs.frankmuenster.de/mahoe/santander\">http://progs.frankmuenster.de/mahoe/santander</a><br/>") //$NON-NLS-1$
				.append("</p></form>"); //$NON-NLS-1$

		final FormTextPart text = new FormTextPart();
		text.setText(formString.toString());
		container.addPart(text);

		setSize(610, 430);

	}

	/**
	 * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
	 */
	@Override
	protected SantanderCreateInfo getData() {

		final CreateAccountInfo info = super.getData();

		data.setOwner(info.getOwner());
		data.setComment(info.getComment());
		data.setDescription(StringUtils.isEmpty(info.getDescription()) ? "Santander Bank" //$NON-NLS-1$
				: info.getDescription());

		return data;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.assistant.AbstractCreateKontoDialog#prepareData()
	 */
	@Override
	protected boolean prepareData() {

		boolean dataOk = true;

		data = new SantanderCreateInfo();

		// Werte speichern
		data.setLogin((String) login.getValue());
		data.setKtoNummer((String) ktoNummer.getValue());
		if (StringUtils.isEmpty(data.getKtoNummer())) {
			data.setKtoNummer(data.getLogin());
		}
		data.setSavePDF((Boolean) savePDF.getValue());
		data.setSaveFolder((String) saveFolder.getValue());
		data.setBankType((SantanderBankEnum) bankTypeInput.getValue());
		if ("Tagesgeldkonto".equals(ktoTypeInput.getValue())) { //$NON-NLS-1$
			data.setKtoType(KontoType.GIRO);
			// Plausibilittsprfung
			if (StringUtils.isEmpty(data.getKtoNummer())
					|| !HBCIProperties.checkAccountCRC(SantanderBank.BLZ, data.getKtoNummer())) {
				dataOk = false;
				ktoNummer.setComment(I18N.tr("SantanderCreateAccountDialog.accountNumberWrong")); //$NON-NLS-1$
			} else {
				ktoNummer.setComment(StringUtils.EMPTY);
			}
		} else {
			data.setKtoType(KontoType.KREDITKARTE);
			// Plausibilittsprfung
			if (StringUtils.isEmpty(data.getKtoNummer()) || !DebitCardValidator.isValidateCardNumber(data.getKtoNummer())) {
				dataOk = false;
				ktoNummer.setComment(I18N.tr("SantanderCreateAccountDialog.creditcardNumberWrong")); //$NON-NLS-1$
			} else {
				ktoNummer.setComment(StringUtils.EMPTY);
			}
		}

		return dataOk;

	}

}
