//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package de.frankmuenster.hibiscus.mahoe.santander.pdfextractor;

import java.io.IOException;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractBankPDFPage;
import de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor;
import de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractTransactionCreator;
import de.frankmuenster.hibiscus.mahoe.pdfextractor.PdfEintrag;
import de.willuhn.logging.Logger;

/**
 * Class for extracting entry information from a TargoBank PDF bank statement.
 * 
 * @author Frank Mnster
 * 
 */
public class SantanderPdfExtractor extends AbstractPdfExtractor {

	/**
	 * 
	 */
	private static final double MAX_POS_DATE_DIFF = 0.05;

	/**
	 * Zeilenhhe der Buchungszeile im MahoeMain PDF.
	 */
	private static final double LINE_HEIGH = 9.35;

	/**
	 * Dezimalformatierer
	 */
	private final static NumberFormat N_FORMAT = NumberFormat.getInstance(Locale.GERMAN);

	/**
	 * Standard constructor
	 */
	public SantanderPdfExtractor() {
		super();
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#removeAnnotationsBookingDates(java.util.List)
	 */
	@Override
	protected void removeAnnotationsBookingDates(final List<PdfEintrag> buchungenDatum) {

		if (!buchungenDatum.isEmpty()) {
			final double xOfFirstDatum = buchungenDatum.get(0).getPositionX();
			Logger.trace("First date: " + xOfFirstDatum); //$NON-NLS-1$
			for (int i = buchungenDatum.size() - 1; i > 0; i--) {
				Logger.trace("Check date x position: " + buchungenDatum.get(i).getPositionX()); //$NON-NLS-1$
				final double xPosDiff = Math.abs(buchungenDatum.get(i).getPositionX() - xOfFirstDatum);
				if (xPosDiff > MAX_POS_DATE_DIFF) {
					Logger.trace("Remove date as it is an annotation in the bottom"); //$NON-NLS-1$
					buchungenDatum.remove(i);
				}
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#removeAnnotations(java.util.List)
	 */
	@Override
	protected void removeAnnotations(final List<PdfEintrag> pdfWortList) {
		final double yOfFirstDatum = buchungsDaten.get(buchungsDaten.size() - 1).getPositionY() - 10;
		Logger.trace("First: " + yOfFirstDatum); //$NON-NLS-1$
		for (int i = pdfWortList.size() - 1; i >= 0; i--) {
			Logger.trace("Check: " + pdfWortList.get(i).getPositionY()); //$NON-NLS-1$

			if (pdfWortList.get(i).getPositionY() < yOfFirstDatum) {
				Logger.trace("Remove"); //$NON-NLS-1$
				pdfWortList.remove(i);
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#addYear
	 */
	@Override
	protected void addYear(final List<PdfEintrag> entries) {

		// Jahr hinzu tun
		final int rMonat = Integer.valueOf(rechungsDatum.substring(3, 5));
		final int rJahr = Integer.valueOf(rechungsDatum.substring(6, 10));

		for (final PdfEintrag entry : entries) {
			final String datum = entry.getWort().substring(0, 2) + "." //$NON-NLS-1$
					+ entry.getWort().substring(2, 4) + "."; //$NON-NLS-1$
			final int bMonat = Integer.valueOf(entry.getWort().substring(2, 4));
			if (bMonat > rMonat) {
				entry.setWort(datum + (rJahr - 1));
			} else {
				entry.setWort(datum + rJahr);
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getTransactionCreator()
	 */
	@Override
	protected AbstractTransactionCreator getTransactionCreator() {
		return new SantanderTransactionCreator(buchungsDaten, LINE_HEIGH);
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getNumberFormat()
	 */
	@Override
	protected NumberFormat getNumberFormat() {
		return N_FORMAT;
	}

	/**
	 * @throws IOException
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getBankPdfFirstPage()
	 */
	@Override
	protected AbstractBankPDFPage getBankPdfFirstPage() throws IOException {
		return new SantanderPdfFirstPage();
	}

	/**
	 * @throws IOException
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getBankPdfEvenPage()
	 */
	@Override
	protected AbstractBankPDFPage getBankPdfEvenPage() throws IOException {
		return new SantanderPdfOtherPage();
	}

	/**
	 * @throws IOException
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getBankPdfOddPage()
	 */
	@Override
	protected AbstractBankPDFPage getBankPdfOddPage() throws IOException {
		return new SantanderPdfOtherPage();
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getBetraege(de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractBankPDFPage)
	 */
	@Override
	protected List<PdfEintrag> getBetraege(final AbstractBankPDFPage textAreas) {

		final List<PdfEintrag> betraege = new ArrayList<>();

		final String regioText = textAreas.getTextForRegion(AbstractBankPDFPage.BETRAG);
		final String[] lines = regioText.split("\n"); //$NON-NLS-1$

		for (final String line : lines) {
			Logger.trace("Line (betrag): >" + line + "<"); //$NON-NLS-1$ //$NON-NLS-2$

			if (StringUtils.isNotEmpty(line)) {
				final String[] woerter = line.split(" "); //$NON-NLS-1$

				if (woerter.length == 0 || woerter.length > MAX_TWO_WORD) {
					break;
				}
				boolean isHaben = false;
				if (woerter.length == MAX_TWO_WORD) {
					// Wenn das zweite Wort nur ein H ist, haben wir eine
					// Einzahlung
					final String[] testHaben = woerter[1].split(WORT_DELIMITER);
					if ("H".equals(testHaben[2].replace("\n", StringUtils.EMPTY))) { //$NON-NLS-1$//$NON-NLS-2$
						isHaben = true;
					}
				}
				final String wort = woerter[0];
				final String[] wortInfo = wort.split(WORT_DELIMITER);
				if (wortInfo.length != WORT_INFO_LENGTH) {
					continue;
				}
				if (!isHaben) {
					wortInfo[2] = '-' + wortInfo[2]; // NOPMD
				}
				final PdfEintrag pdfEintrag = new PdfEintrag(); // NOPMD
				pdfEintrag.setPositionX(Double.valueOf(wortInfo[0]));
				pdfEintrag.setPositionY(Double.valueOf(wortInfo[1]));
				pdfEintrag.setWort(wortInfo[2].replace("\n", StringUtils.EMPTY)); //$NON-NLS-1$
				pdfEintrag.setWort(pdfEintrag.getWort().replace("\r", StringUtils.EMPTY)); //$NON-NLS-1$
				betraege.add(pdfEintrag);
			}
		}

		// Anmerkungen entfernen
		removeAnnotations(betraege);

		return betraege;
	}

}
