//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
//

package de.frankmuenster.hibiscus.mahoe.targobank.pdfextractor;

import java.io.IOException;
import java.text.NumberFormat;
import java.util.List;
import java.util.Locale;

import de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractBankPDFPage;
import de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor;
import de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractTransactionCreator;
import de.frankmuenster.hibiscus.mahoe.pdfextractor.PdfEintrag;
import de.willuhn.logging.Logger;

/**
 * Class for extracting entry information from a TargoBank PDF bank statement.
 * 
 * @author Frank Mnster
 * 
 */
public class TargoBankPdfExtract extends AbstractPdfExtractor {

	/**
	 * Zeilenhhe der Buchungszeile im MahoeMain PDF.
	 */
	private static final double LINE_HEIGH = 9.35;

	/**
	 * Dezimalformatierer
	 */
	private final static NumberFormat N_FORMAT = NumberFormat.getInstance(Locale.GERMAN);

	/**
	 * Standard constructor
	 */
	public TargoBankPdfExtract() {
		super();
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#removeAnnotationsBookingDates(java.util.List)
	 */
	@Override
	protected void removeAnnotationsBookingDates(final List<PdfEintrag> buchungenDatum) {

		if (!buchungenDatum.isEmpty()) {
			final double xOfFirstDatum = buchungenDatum.get(0).getPositionX();
			Logger.trace("First date: " + xOfFirstDatum); //$NON-NLS-1$
			for (int i = buchungenDatum.size() - 1; i > 0; i--) {
				Logger.trace("Check date x position: " + buchungenDatum.get(i).getPositionX()); //$NON-NLS-1$

				if (buchungenDatum.get(i).getPositionX() != xOfFirstDatum) {
					Logger.trace("Remove date as it is an annotation in the bottom"); //$NON-NLS-1$
					buchungenDatum.remove(i);
				}
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#removeAnnotations(java.util.List)
	 */
	@Override
	protected void removeAnnotations(final List<PdfEintrag> pdfWortList) {
		final double yOfFirstDatum = buchungsDaten.get(buchungsDaten.size() - 1).getPositionY() - 10;
		Logger.trace("First: " + yOfFirstDatum); //$NON-NLS-1$
		for (int i = pdfWortList.size() - 1; i >= 0; i--) {
			Logger.trace("Check: " + pdfWortList.get(i).getPositionY()); //$NON-NLS-1$

			if (pdfWortList.get(i).getPositionY() < yOfFirstDatum) {
				Logger.trace("Remove"); //$NON-NLS-1$
				pdfWortList.remove(i);
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#addYear
	 */
	@Override
	protected void addYear(final List<PdfEintrag> entries) {

		// Jahr hinzu tun
		final int rMonat = Integer.valueOf(rechungsDatum.substring(3, 5));
		final int rJahr = Integer.valueOf(rechungsDatum.substring(6, 10));

		for (final PdfEintrag entry : entries) {

			final int bMonat = Integer.valueOf(entry.getWort().substring(3, 5));
			if (bMonat > rMonat) {
				entry.setWort(entry.getWort() + (rJahr - 1));
			} else {
				entry.setWort(entry.getWort() + rJahr);
			}
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getTransactionCreator()
	 */
	@Override
	protected AbstractTransactionCreator getTransactionCreator() {
		return new TargoBankTransactionCreator(buchungsDaten, LINE_HEIGH);
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getNumberFormat()
	 */
	@Override
	protected NumberFormat getNumberFormat() {
		return N_FORMAT;
	}

	/**
	 * @throws IOException
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getBankPdfFirstPage()
	 */
	@Override
	protected AbstractBankPDFPage getBankPdfFirstPage() throws IOException {
		return new TargoBankPdfFirstPage();
	}

	/**
	 * @throws IOException
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getBankPdfEvenPage()
	 */
	@Override
	protected AbstractBankPDFPage getBankPdfEvenPage() throws IOException {
		return new TargoBankPdfEvenPage();
	}

	/**
	 * @throws IOException
	 * @see de.frankmuenster.hibiscus.mahoe.pdfextractor.AbstractPdfExtractor#getBankPdfOddPage()
	 */
	@Override
	protected AbstractBankPDFPage getBankPdfOddPage() throws IOException {
		return new TargoBankPdfOddPage();
	}

}
