//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.targobank.web; // NOPMD 

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.NicelyResynchronizingAjaxController;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.WebClientOptions;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.DomNode;
import com.gargoylesoftware.htmlunit.html.DomNodeList;
import com.gargoylesoftware.htmlunit.html.HtmlAnchor;
import com.gargoylesoftware.htmlunit.html.HtmlElement;
import com.gargoylesoftware.htmlunit.html.HtmlEmphasis;
import com.gargoylesoftware.htmlunit.html.HtmlForm;
import com.gargoylesoftware.htmlunit.html.HtmlInput;
import com.gargoylesoftware.htmlunit.html.HtmlOption;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlSelect;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTableBody;
import com.gargoylesoftware.htmlunit.html.HtmlTableRow;

import de.frankmuenster.hibiscus.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.hibiscus.mahoe.targobank.TargoBank;
import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.frankmuenster.hibiscus.mahoe.utils.Utils;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaver;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaverException;
import de.frankmuenster.hibiscus.mahoe.utils.pdfsaver.PdfSaverExceptionTyp;
import de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteException;
import de.frankmuenster.hibiscus.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Stellt das Webseiten-Interface dar.
 * 
 * @author Frank Mnster
 * 
 */
public class TargoBankWebSite extends AbstractWebSite<TargoBankAccountInformation, TargoBankMessages> { // NOPMD

	/**
	 * Seite mit den PDF-Abrechnungen nicht gefunden
	 */
	protected final static String PDFSITE_NOT_FOUND = "Abrechnungen konnten nicht ge\u00F6ffnet werden"; //$NON-NLS-1$

	/**
	 * Das Konto
	 */
	private final transient Konto konto;

	/**
	 * Das Implementierungs-Objekt fr die Umstze
	 */
	private final transient TargoBankWebSiteTransactionsImpl transImpl;

	/**
	 * Das Login/die Kundenkennung
	 */
	private final transient String kundenkennung;

	/**
	 * Unsere BceeWebSite instance
	 */
	private static TargoBankWebSite webSite;

	/**
	 * Liefert die Instance der WebSeite
	 * 
	 * @param konto
	 * @return die WebSeite
	 * @throws WebSiteException
	 * @throws ApplicationException
	 */
	public static TargoBankWebSite getInstance(final Konto konto) throws WebSiteException, ApplicationException {

		String cardAccount;
		String cardNumber;
		String cardCompany;
		String loginName;

		try {
			loginName = konto.getKundennummer();
			if (StringUtils.isEmpty(loginName)) {
				throw new ApplicationException(I18N.tr("TargoBankWebSite.loginMissing")); //$NON-NLS-1$
			}

			cardAccount = konto.getMeta(MahoeSynchronizeBackend.CARD_ACCOUNT, null);
			if (StringUtils.isEmpty(cardAccount)) {
				throw new ApplicationException(I18N.tr("TargoBankWebSite.cardAccountMissing")); //$NON-NLS-1$
			}

			cardNumber = konto.getMeta(MahoeSynchronizeBackend.CARD_NUMBER, null);
			if (StringUtils.isEmpty(cardNumber)) {
				throw new ApplicationException(I18N.tr("TargoBankWebSite.cardNumberMissing")); //$NON-NLS-1$
			}

			cardNumber = cardNumber.replaceAll("\\s+", StringUtils.EMPTY); //$NON-NLS-1$
			cardCompany = konto.getMeta(MahoeSynchronizeBackend.CARD_COMPANY, null);
			if (StringUtils.isEmpty(cardCompany)) {
				throw new ApplicationException(I18N.tr("TargoBankWebSite.cardCompanyMissing")); //$NON-NLS-1$
			}

			if (TargoBankWebSite.webSite == null) { // NOPMD
				TargoBankWebSite.webSite = new TargoBankWebSite(konto, loginName, cardCompany, cardAccount, cardNumber);
			} else {
				// Ist das Konto dasselbe
				if (TargoBankWebSite.webSite.konto.equals(konto)) {
					// Aktualisiere Startdatum der Umsatzabfrage
					final Date lastTransacDate = Utils
							.getLastUmsatzDate(konto, TargoBankWebSite.webSite.getStartdate());
					TargoBankWebSite.webSite.accountInfo.setLastTransactionDate(lastTransacDate);
				} else {
					TargoBankWebSite.webSite.logoff();
					TargoBankWebSite.webSite = new TargoBankWebSite(konto, loginName, cardCompany, cardAccount,
							cardNumber);
				}
			}
		} catch (RemoteException e) {
			final String msg = I18N.tr("TargoBankWebSite.readAccountInfoFailed"); //$NON-NLS-1$
			Logger.error(msg, e);
			throw new ApplicationException(msg, e);
		}
		return TargoBankWebSite.webSite;
	}

	/**
	 * Erstellt ein neues Webseiten-Objekt zur Kontaktaufnahme mit der
	 * TargoBank.
	 * 
	 * @param konto
	 * @param login
	 * @param cardCompany
	 * @param cardAccount
	 * @param cardNumber
	 * 
	 * @throws WebSiteException
	 */
	public TargoBankWebSite(final Konto konto, final String login, final String cardCompany, final String cardAccount,
			final String cardNumber) throws WebSiteException {
		super(konto);

		this.konto = konto;
		this.kundenkennung = login;

		this.accountInfo.setCardCompany(cardCompany);
		this.accountInfo.setCardNumber(cardNumber);
		this.accountInfo.setCardAccount(cardAccount);

		this.transImpl = new TargoBankWebSiteTransactionsImpl(this.webClient, this.accountInfo);

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#login(char[],
	 *      String...)
	 */
	@Override
	public void login(final char[] password, final String... loginParams) throws WebSiteException {

		if (loginParams.length != 0) {
			throw new WebSiteException(I18N.tr("TargoBankWebSite.loginParameterNumberWrong"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_ERROR);
		}
		try {
			final HtmlPage pageLogin = this.webClient.getPage(TargoBank.URL_LOGIN);
			Logger.trace(pageLogin.asText());

			final HtmlForm formLogin = pageLogin.getFormByName("bloc_ident"); //$NON-NLS-1$
			formLogin.getInputByName("_cm_user").setValueAttribute(this.kundenkennung); //$NON-NLS-1$
			formLogin.getInputByName("_cm_pwd").setValueAttribute(new String(password)); //$NON-NLS-1$
			final HtmlInput submitContinue = formLogin.getInputByName("submit"); //$NON-NLS-1$
			Logger.info(I18N.tr("TargoBankWebSite.sendLoginData")); //$NON-NLS-1$

			final HtmlPage pageLogin2 = submitContinue.click();
			Logger.debug("Seite: " + pageLogin2.asText()); //$NON-NLS-1$

			// Erster Test (Negativ Text ist vorhanden)
			if (pageLogin2.asText().contains(
					"Die von Ihnen eingegebenen Daten sind falsch. Bitte versuchen Sie es erneut.")) { //$NON-NLS-1$
				Logger.warn(I18N.tr("TargoBankWebSite.loginDataWrong")); //$NON-NLS-1$
				throw new WebSiteException(I18N.tr("TargoBankWebSite.loginDataWrong"), //$NON-NLS-1$
						WebSiteExceptionTypes.LOGIN_FAILED);
			}

			// Zweiter Test (positiv Text/Element ist nicht vorhanden)
			final DomElement logoutBtn = pageLogin2.getElementById("ei_tpl_ident_logout"); //$NON-NLS-1$
			if (logoutBtn == null) {
				Logger.warn(I18N.tr("TargoBankWebSite.logoutBtnNotFound")); //$NON-NLS-1$
				throw new WebSiteException(I18N.tr("TargoBankWebSite.logoutBtnNotFound"), //$NON-NLS-1$
						WebSiteExceptionTypes.LOGIN_FAILED);
			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(I18N.tr("TargoBankWebSite.loginFailedLog", e.getMessage())); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("TargoBankWebSite.loginFailed"), //$NON-NLS-1$
					WebSiteExceptionTypes.LOGIN_ERROR, e);
		}
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#getAccountInformation()
	 */
	@Override
	public TargoBankAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

		try {
			final HtmlPage page = TargoBankWebSiteImpl.openAccount(this.webClient, this.accountInfo);

			final DomElement section = page.getElementById("page"); //$NON-NLS-1$
			final DomNodeList<HtmlElement> tables = section.getElementsByTagName(HtmlTable.TAG_NAME);

			final SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy", Locale.getDefault()); //$NON-NLS-1$
			for (final HtmlElement element : tables) {
				if (element instanceof HtmlTable) {
					final HtmlTable table = (HtmlTable) element;
					Logger.trace("Table summery: " + table.getSummaryAttribute()); //$NON-NLS-1$
					if (table.getSummaryAttribute().equals("Card Summary")) { //$NON-NLS-1$
						// hole Grundinfos
						String value;
						final HtmlTableBody body = table.getBodies().get(0);
						// Saldo
						value = body.getRows().get(0).getCell(1).getTextContent().trim().replaceAll("\u00a0", //$NON-NLS-1$
								StringUtils.EMPTY);
						this.accountInfo.setSaldo(Utils.convertStringToDouble(value));
						// Verfgungsrahmen
						value = body.getRows().get(1).getCell(1).getTextContent().trim().replaceAll("\u00a0", //$NON-NLS-1$
								StringUtils.EMPTY);
						this.accountInfo.setVerfuegungsrahmen(Utils.convertStringToDouble(value));
						// Verfgbarer Betrag
						value = body.getRows().get(2).getCell(1).getTextContent().trim().replaceAll("\u00a0", //$NON-NLS-1$
								StringUtils.EMPTY);
						this.accountInfo.setSaldoAvailable(Utils.convertStringToDouble(value));
						// Nchster Flligkeitstermin
						value = body.getRows().get(2).getCell(3).getTextContent().trim().replaceAll("\u00a0", //$NON-NLS-1$
								StringUtils.EMPTY);
						Date nextPayday = null;
						try {
							nextPayday = sdf.parse(value);
						} catch (ParseException e) {
							Logger.error(I18N.tr("TargoBankWebSite.parsePayDayFailed", e.getMessage()), e); //$NON-NLS-1$
						}
						this.accountInfo.setNextPayDay(nextPayday);

						Logger.info(I18N.tr("TargoBankWebSite.logSaldo") + this.accountInfo.getSaldo()); //$NON-NLS-1$
						Logger.info(I18N.tr("TargoBankWebSite.logSalsoAvailable") + this.accountInfo.getSaldoAvailable()); //$NON-NLS-1$
						Logger.info(I18N.tr("TargoBankWebSite.logVerfuegungsrahmen") + this.accountInfo.getVerfuegungsrahmen()); //$NON-NLS-1$
						Logger.info(I18N.tr("TargoBankWebSite.logPayDay") + this.accountInfo.getSaldo()); //$NON-NLS-1$
						break;
					}
				}
			}
		} catch (WebSiteException e) {
			Logger.error(I18N.tr("TargoBankWebSite.readAccountOverviewFailedLog", e.getMessage()), e); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("TargoBankWebSite.readAccountOverviewFailed"), // NOPMD //$NON-NLS-1$
					WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
		}

		return this.accountInfo;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#getPreBookedTransactions()
	 */
	@Override
	public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {

		return this.transImpl.getPreBookTransactions();

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#getBookedTransactions()
	 */
	@Override
	public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

		return this.transImpl.getBookedTransactions();
	}

	/**
	 * Speichert die Kontoauszge im angegeben Ordner
	 * 
	 * @throws PdfSaverException
	 */
	@Override
	public int savePDF(final TargoBankAccountInformation info) throws PdfSaverException {

		int savedCount = 0;
		HtmlPage pageAbr;
		try {
			pageAbr = this.webClient.getPage(TargoBank.URL_ABBRECHNUNGEN);
			if (pageAbr.asText().contains("Online-Postfach")) { //$NON-NLS-1$

				// Richtige Karte auswaehlen
				final String kartenId = "TARGO   0000" + this.accountInfo.getCardNumber(); //$NON-NLS-1$

				final HtmlSelect selIndex1 = pageAbr.getElementByName("SelIndex1"); //$NON-NLS-1$
				final HtmlInput submit = (HtmlInput) pageAbr.getElementById("btnRechercher"); //$NON-NLS-1$

				final HtmlOption option = selIndex1.getOptionByValue(kartenId);
				selIndex1.setSelectedAttribute(option, true);

				HtmlPage pageNew = submit.click();
				boolean hasNext;

				final PdfSaver pdfSaver = new PdfSaver(info.getSavePath());

				do {

					final int saved = this.savePdfs(pageNew, pdfSaver);
					savedCount += saved;

					hasNext = false;
					final HtmlTable table = (HtmlTable) pageNew.getElementById("Table1"); //$NON-NLS-1$
					final DomNodeList<HtmlElement> inputButtons = table.getElementsByTagName(HtmlInput.TAG_NAME);
					for (final HtmlElement htmlElement : inputButtons) {
						if (htmlElement.getId().equals("NEXT")) { //$NON-NLS-1$
							hasNext = true;
							pageNew = htmlElement.click();
							break;
						}
					}

				} while (hasNext);

			} else {
				Logger.warn(TargoBankWebSite.PDFSITE_NOT_FOUND);
				throw new PdfSaverException(TargoBankWebSite.PDFSITE_NOT_FOUND, PdfSaverExceptionTyp.PDFSITE_NOT_FOUND);
			}
		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(I18N.tr("TargoBankWebSite.loadPdfFailedLog", e.getMessage()), e); //$NON-NLS-1$
			throw new PdfSaverException(I18N.tr("TargoBankWebSite.loadPdfFailed"), e); //$NON-NLS-1$

		}

		return savedCount;
	}

	/**
	 * Speichert die Kontoauszge
	 * 
	 * @param page
	 * @param pdfSaver
	 */
	private final int savePdfs(final HtmlPage page, final PdfSaver pdfSaver) { // NOPMD

		int saveCount = 0;

		final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", Locale.GERMAN); //$NON-NLS-1$
		final SimpleDateFormat sdfFileName = new SimpleDateFormat("yyyy_MM(MMMM)", Locale.GERMAN); //$NON-NLS-1$

		final HtmlTable table = (HtmlTable) page.getElementById("Table1"); //$NON-NLS-1$
		final DomNodeList<HtmlElement> tables = table.getElementsByTagName(HtmlTable.TAG_NAME);

		StringBuffer fileName = new StringBuffer(18);

		final HtmlTableBody tbBody = ((HtmlTable) tables.item(1)).getBodies().get(0);
		final List<HtmlTableRow> tbRows = tbBody.getRows();
		for (final HtmlTableRow tbRow : tbRows) {
			final String pdfDateString = tbRow.getCell(1).getTextContent().trim().replaceAll("\u00a0", //$NON-NLS-1$
					StringUtils.EMPTY);
			Logger.debug("Abrechnungsdatum: " + pdfDateString); //$NON-NLS-1$

			try {
				final Date pdfDate = sdf.parse(pdfDateString);
				fileName = new StringBuffer(sdfFileName.format(pdfDate)); // NOPMD

				final DomNode urlText = tbRow.getCell(2).getFirstChild();
				Logger.debug("URLtext: " + urlText); //$NON-NLS-1$

				HtmlAnchor anchor = null;
				if (urlText instanceof HtmlAnchor) {
					anchor = (HtmlAnchor) urlText;
					Logger.debug("Alte Abrechnung gefunden."); //$NON-NLS-1$
				} else if (urlText instanceof HtmlEmphasis) {
					Logger.debug("Neue Abrechnung gefunden."); //$NON-NLS-1$
					anchor = (HtmlAnchor) urlText.getFirstChild();
				} else {
					Logger.error(I18N.tr("TargoBankWebSite.getPdfUrlFailed")); //$NON-NLS-1$
				}
				if (anchor != null && fileName != null) {
					final URL fullUrl = page.getFullyQualifiedUrl(anchor.getHrefAttribute());
					anchor.setAttribute("href", fullUrl.toString()); //$NON-NLS-1$
					Logger.debug("URL: " + anchor.getHrefAttribute()); //$NON-NLS-1$
					fileName = anchor.getHrefAttribute().contains("Valovis") ? fileName.append("_Valovis.pdf") //$NON-NLS-1$ //$NON-NLS-2$
							: fileName.append(".pdf"); //$NON-NLS-1$
					if (pdfSaver.savePdf(fileName.toString(), anchor)) {
						saveCount++;
					}
				}

			} catch (MalformedURLException e) {
				Logger.error(I18N.tr("TargoBankWebSite.pdfUrlMalforment", e.getMessage()), e); //$NON-NLS-1$
			} catch (ParseException e) {
				Logger.error(I18N.tr("TargoBankWebSite.parsePdfDateFailed", e.getMessage()), e); //$NON-NLS-1$
			} catch (PdfSaverException e) {
				Logger.error(I18N.tr("TargoBankWebSite.savePdfFailed", e.getMessage()), e); //$NON-NLS-1$
			}
		} // end for tbRows
		return saveCount;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#logoff()
	 */
	@Override
	public void logoff() throws WebSiteException {

		try {
			final HtmlPage logoff = this.webClient.getPage(TargoBank.URL_LOGOFF);
			if (logoff.asText().contains("Erfolgreich ausgeloggt.")) { //$NON-NLS-1$
				Logger.info(I18N.tr("TargoBankWebSite.logoffSuccessful")); //$NON-NLS-1$
			} else {
				throw new WebSiteException(I18N.tr("TargoBankWebSite.logoffFailed"), //$NON-NLS-1$
						WebSiteExceptionTypes.LOGOFF_FAILED);
			}

		} catch (FailingHttpStatusCodeException | IOException e) {
			Logger.error(I18N.tr("TargoBankWebSite.logoffFailed"), e); //$NON-NLS-1$
			throw new WebSiteException(I18N.tr("TargoBankWebSite.logoffFailed"), WebSiteExceptionTypes.LOGOFF_FAILED, //$NON-NLS-1$
					e);
		}

	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite#getWebClient()
	 */
	@Override
	protected WebClient getWebClient() {

		final WebClient webcl = new WebClient();

		final WebClientOptions options = webcl.getOptions();

		options.setUseInsecureSSL(false);
		options.setJavaScriptEnabled(true);
		options.setThrowExceptionOnFailingStatusCode(false);
		options.setCssEnabled(false);
		options.setThrowExceptionOnScriptError(false);
		options.setRedirectEnabled(true);

		webcl.setAjaxController(new NicelyResynchronizingAjaxController());
		webcl.getCookieManager().setCookiesEnabled(true);

		Utils.setProxyCfg(webcl, TargoBank.URL_LOGIN);

		return webcl;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.AbstractWebSite#getStartdate()
	 */
	@Override
	protected Calendar getStartdate() {

		// Setzte das startDatum auf Januar 2016
		final Calendar startCalendar = Calendar.getInstance();
		startCalendar.set(2016, 0, 1);
		return startCalendar;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#isStillLoggedIn()
	 */
	@Override
	public boolean isStillLoggedIn() throws WebSiteException {
		return false;
	}

	/**
	 * @see de.frankmuenster.hibiscus.mahoe.web.InterfaceWebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
	 */
	@Override
	public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung) throws WebSiteException,
			ApplicationException {
		Logger.info(I18N.tr("TargoBankWebSite.transferNotSupported")); //$NON-NLS-1$

	}

}
