//
// Copyright 2016 frankmuenster.de
// 
// Lizenziert unter der EUPL, Version 1.1 oder - sobald
// diese von der Europischen Kommission genehmigt wurden -
// Folgeversionen der EUPL ("Lizenz");
// Sie drfen dieses Werk ausschlielich gem
// dieser Lizenz nutzen.
// Eine Kopie der Lizenz finden Sie hier:
// 
//   http://ec.europa.eu/idabc/eupl
// 
// Sofern nicht durch anwendbare Rechtsvorschriften
// gefordert oder in schriftlicher Form vereinbart, wird
// die unter der Lizenz verbreitete Software "so wie sie
// ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
// ausdrcklich oder stillschweigend - verbreitet.
// Die sprachspezifischen Genehmigungen und Beschrnkungen
// unter der Lizenz sind dem Lizenztext zu entnehmen.
// 

package de.frankmuenster.hibiscus.mahoe.utils.pdfsaver;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;

import com.gargoylesoftware.htmlunit.html.HtmlAnchor;

import de.frankmuenster.hibiscus.mahoe.utils.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.logging.Logger;

/**
 * Kmmert sich um das Speichern der Kontoauszge
 * 
 * @author Frank Mnster
 * 
 */
public final class PdfSaver {

	/**
	 * Speicherpfad
	 */
	private final transient File pdfFolder;

	/**
	 * Der OrdnerPath als Rckgabe vom DirectoryDialog. Muss klassenweit gesetzt
	 * werden, da der Dialog in einem eigenstndigem Threat luft.
	 */
	private static String path;

	/**
	 * Konstruktor mit Ordnername. Ordner wird erstellt, falls nicht vorhanden.
	 * 
	 * @param folder
	 *            der Ordnername
	 * 
	 * @throws PdfSaverException
	 *             falls der Ordner nicht erstellt werden konnte.
	 */
	public PdfSaver(final String folder) throws PdfSaverException {

		this.pdfFolder = new File(folder);
		if (!this.pdfFolder.isAbsolute()) {
			throw new PdfSaverException(I18N.tr("PdfSaver.folderMustBeAbsolute")); //$NON-NLS-1$
		}
		// Gibt es den Ordner
		if (!this.pdfFolder.getParentFile().exists()) {
			if (!this.pdfFolder.getParentFile().mkdir()) { // NOPMD
				throw new PdfSaverException(I18N.tr(
						"PdfSaver.mkdirFolderFailed", this.pdfFolder.getParentFile().toString()), //$NON-NLS-1$
						PdfSaverExceptionTyp.FOLDER_IS_FILE);
			}
		}
	}

	/**
	 * Speichert alle noch nicht gespeicherten Kontoauszge im angegeben Ordner.
	 * ACHTUNG: Wird ein neuer Ordner angegeben, werden alle Auszge erneut
	 * gespeichert.
	 * 
	 * @param fileName
	 *            der Dateiname des PDF
	 * @param link
	 *            das HtmlAnchoer Object mit dem Link zum PDF auf der Webseite
	 * 
	 * @throws PdfSaverException
	 */
	public boolean savePdf(final String fileName, final HtmlAnchor link) throws PdfSaverException {
		boolean saved = false;
		// Check file exists
		final File pdfFile = new File(this.pdfFolder.getAbsolutePath() + "/" + fileName); //$NON-NLS-1$
		if (!pdfFile.exists()) {
			this.download(pdfFile.getAbsolutePath(), link);
			Logger.info(I18N.tr("PdfSaver.pdfSaved", fileName)); //$NON-NLS-1$
			saved = true;
		}
		return saved;
	}

	/**
	 * downloads <i>link</i> and save with <i>localFileName</i>
	 * 
	 * @param link
	 *            HtmlAnchor to file to download
	 * @param localFileName
	 *            full file and path name for file to save
	 */
	private void download(final String localFileName, final HtmlAnchor link) {

		OutputStream outputStream = null;
		InputStream inputStream = null;

		try {
			inputStream = link.click().getWebResponse().getContentAsStream();
			outputStream = new BufferedOutputStream(new FileOutputStream(localFileName));
			final byte[] buffer = new byte[1024];

			int numRead;
			long numWritten = 0;

			while ((numRead = inputStream.read(buffer)) != -1) { // NOPMD
				outputStream.write(buffer, 0, numRead);
				numWritten += numRead;
			}
			Logger.debug(localFileName + "\t" + numWritten); //$NON-NLS-1$
		} catch (FileNotFoundException e) {
			Logger.error(I18N.tr("PdfSaver.fileNameWrong", e.getMessage())); //$NON-NLS-1$
		} catch (IOException exception) {
			exception.printStackTrace(System.out);
			Logger.error(exception.getMessage());
		} catch (Exception e) { // NOPMD
			e.printStackTrace(System.out);
			Logger.error(e.getMessage());

		} finally {
			try {
				if (inputStream != null) {
					inputStream.close();
				}
				if (outputStream != null) {
					outputStream.close();
				}
			} catch (IOException ioe) { // NOPMD
				// jetzt geben wir auf :-)
			}
		}
	}

	/**
	 * Opens a DirectoryDialog. Returns the chosen/typed file with full path.
	 * 
	 * @return file name with full path name
	 */
	public static String getPath() {

		// final Display display = GUI.getDisplay();
		Display.getDefault().syncExec(new Runnable() { // NOPMD
					/**
					 * 
					 * @see java.lang.Runnable#run()
					 */
					@Override
					public void run() {
						final DirectoryDialog dlg = new DirectoryDialog(GUI.getShell());

						// Change the title bar text
						dlg.setText(I18N.tr("PdfSaver.saveFolderDialog")); //$NON-NLS-1$

						// Customisable message displayed in the dialog
						dlg.setMessage(I18N.tr("PdfSaver.saveFolderDialogMsg")); //$NON-NLS-1$

						// Calling open() will open and run the dialog.
						// It will return the selected directory, or
						// null if user cancels
						PdfSaver.path = dlg.open();

					}
				});

		return PdfSaver.path;

	}

}
